<?php


namespace app\cbcrm\controller\api;


use app\cbcrm\model\config\SettingModel;
use app\cbcrm\model\copyright\OrderModel;
use app\cbcrm\model\copyright\ProductModel;
use app\cbcrm\model\member\MemberModel;
use app\cbcrm\services\SmsServices;
use EasyWeChat\Factory;
use think\App;
use think\Exception;
use think\facade\Db;

class CopyrightOrder extends Base
{
    public function __construct(App $app)
    {
        parent::__construct($app);
        $this->auth = $this->auth();
    }

    /*
     * 用户中心订单列表
     * */
    public function userOrderList()
    {
        $pay_status = $this->request->get('pay_status', -1);
        $where = [
            ['wid', '=', $this->wid],
            ['user_id', '=', $this->auth->id],
        ];
        if ($pay_status != -1) {
            $where[] = ['pay_status', '=', $pay_status];
        }
        $list = (new OrderModel())
            ->where($where)
            ->order('create_time', 'desc')
            ->paginate(15);
        $this->success('返回成功', $list);
    }

    /*
     * 用户订单详情
     * */
    public function userOrderDetail()
    {
        $id = $this->request->get('id');
        $where = [
            ['wid', '=', $this->wid],
            ['user_id', '=', $this->auth->id],
            ['id', '=', $id],
        ];

        $row = (new OrderModel())->with(['product'])->where($where)->find();

        if (!$row || $row->user_id != $this->auth->id) {
            $this->error('订单未找到');
        }

        $row->user = $this->auth;

        if ($row->material_list) {
            $material_list = json_decode($row->material_list, true);
            foreach ($material_list as $key => $item) {
                $material_list[$key]['url'] = cdnurl($item['url'], true);
            }
            $row->material_list = $material_list;
        }
        if ($row->certificate) {
            $row->certificate = cdnurl($row->certificate, true);
        }

        $this->success('返回成功', $row);
    }

    /*
     * 软著材料提交
     * */
    public function softwareMaterial()
    {
        $order_id = $this->request->post('order_id');
        $step = $this->request->post('step');
        $where = [
            ['wid', '=', $this->wid],
            ['user_id', '=', $this->auth->id],
            ['id', '=', $order_id],
        ];

        $row = (new OrderModel())->where($where)->find();

        if (!$row || $row->user_id != $this->auth->id) {
            $this->error('订单未找到');
        }

        if ($row->pay_status != 1) {
            $this->error('当前订单未付款');
        }
        if ($row->status != 0) {
            $this->error('当前订单状态不允许该操作');
        }

        $material = Db::table('cb_crm_copyright_software_materials')->where('order_id', $row->id)->find();

        $params = $this->request->post();
        $res = false;
        Db::startTrans();
        try {
            if ($step == 1) {

                if (!$params['softwareFullName']) {
                    $this->error('请输入软件全称');
                }
                if (!$params['version']) {
                    $this->error('请输入软件版本号');
                }

                if ($params['isReg'] == '1' && !$params['inheritanceOrigRegNumber']) {
                    $this->error('请输入正确的原登记号');
                }
                if ($params['isReg'] == '1' && !$params['inheritanceOrigRegPaper']) {
                    $this->error('请上传原登记证书扫描件');
                }
                if ($params['isChange'] == '1' && !$params['inheritanceOrigChangeNumber']) {
                    $this->error('请输入变更或补充证明编号');
                }
                if ($params['rightScope'] == 2 && !$params['rightScopePart']) {
                    $this->error('请选择权利范围');
                }
                if ($params['rightScope'] == 2 && !$params['rightScopePartPaper']) {
                    $this->error('请上传权利范围相关文件');
                }
                $params['user_id'] = $this->auth->id;
                $params['order_id'] = $row->id;
                $params['isDraft'] = 1;
                if ($material) {
                    // 存在则更新
                    $params['update_time'] = date('Y-m-d H:i:s');
                    $res = Db::table('cb_crm_copyright_software_materials')->where('id', $material['id'])->update($params);
                } else {
                    // 新增
                    $params['create_time'] = date('Y-m-d H:i:s');
                    $params['update_time'] = date('Y-m-d H:i:s');
                    $res = Db::table('cb_crm_copyright_software_materials')->insert($params);
                }
            } elseif ($step == 2) {
                if ($params['softwareDescription'] == 'MODIFY' && !$params['modifyDescription']) {
                    $this->error('请填写修改合成或翻译说明');
                }
                if ((isset($params['isReg']) && $params['isReg'] == 1) && (!$params['origRegisterNumber'] || !$params['origRegisterPaper'])) {
                    $this->error('请填写原有软件登记号和上传原登记证书扫描件');
                }
                if (isset($params['isModifyAuth']) && $params['isModifyAuth'] == 1 && !$params['modifyAuthPaper']) {
                    $this->error('请上传授权书');
                }
                if ($params['developMode'] != '单独开发' && !$params['cooperateContract']) {
                    $this->error('请上传合作开发/委托/项目合同');
                }
                if ($params['publishStatus'] == 1 && (!$params['appearDate'] || !$params['appearCity'])) {
                    $this->error('请填写发表日期和首次发表地点');
                }
                $num = 1;
                $ownersData = [];
                $owners = json_decode(htmlspecialchars_decode(html_entity_decode($params['owners'])), true);
                foreach ($owners as $key => $owner) {
                    !$owner['cityName'] ? $this->error("请选择第{$num}个著作权人地区信息") : true;
                    !$owner['peopleType'] ? $this->error("请选择第{$num}个著作权人身份类别") : true;
                    !$owner['name'] ? $this->error("请填写第{$num}个著作权人姓名或名称") : true;
                    !$owner['idType'] ? $this->error("请选择第{$num}个著作权人证件类型") : true;
                    !$owner['idNumber'] ? $this->error("请填写第{$num}个著作权人证件号码") : true;
                    $num++;
                    array_push($ownersData, $owner);
                }
                $params['owners'] = json_encode($ownersData, true);

                $params['update_time'] = date('Y-m-d H:i:s');
                $res = Db::table('cb_crm_copyright_software_materials')->where('id', $material['id'])->update($params);
            } elseif ($step == 3) {
                $params['hardwareDevelopEnv'] ?: $this->error('请填写开发的硬件环境');
                $params['hardwareRunningEnv'] ?: $this->error('请填写运行的硬件环境');
                $params['softwareDevelopSys'] ?: $this->error('请填写开发该软件的操作系统');
                $params['softwareRunningSys'] ?: $this->error('请填写软件开发环境或开发工具');
                $params['softwareDevelopEnv'] ?: $this->error('请填写软件运行平台/操作系统');
                $params['softwareRunningEnv'] ?: $this->error('请填写软件运行支撑环境/支持软件');
                $params['programLanguage'] ?: $this->error('请选择编程语言');

                $params['owners'] = json_decode(htmlspecialchars_decode(html_entity_decode($params['owners'])), true);

                $programIdentifyMaterial = json_decode(htmlspecialchars_decode(html_entity_decode($params['programIdentifyMaterial'])), true);
                $documentIdentifyMaterial = json_decode(htmlspecialchars_decode(html_entity_decode($params['documentIdentifyMaterial'])), true);
                $params['update_time'] = date('Y-m-d H:i:s');
                $params['programIdentifyMaterial'] = json_encode($programIdentifyMaterial, true);
                $documentIdentifyMaterialData = [];
                foreach ($documentIdentifyMaterial as $item) {
                    array_push($documentIdentifyMaterialData, $item);
                }
                $params['programLanguage'] = trim($params['programLanguage'] . ',' . $params['programLanguageOther'], ',');
                $params['documentIdentifyMaterial'] = json_encode($documentIdentifyMaterialData, true);
                $params['updatetime'] = time();
                $params['isDraft'] = 0;
                $res = Db::table('cb_crm_copyright_software_materials')->where('id', $material['id'])->update($params);
                $row->save(['status' => 1]);
            }
            Db::commit();
        } catch (Exception $e) {
            Db::rollback();
            $this->error($e->getMessage());
        }
        if ($res !== false) {
            $this->success('材料提交成功');
        } else {
            $this->error('材料提交失败');
        }

    }

    /*
     * 作品著作权材料提交
     * */
    public function opusMaterial()
    {
        $order_id = $this->request->post('order_id');
        $step = $this->request->post('step');
        $where = [
            ['wid', '=', $this->wid],
            ['user_id', '=', $this->auth->id],
            ['id', '=', $order_id],
        ];

        $row = (new OrderModel())->where($where)->find();

        if (!$row || $row->user_id != $this->auth->id) {
            $this->error('订单未找到');
        }

        if ($row->pay_status != 1) {
            $this->error('当前订单未付款');
        }
        if ($row->status != 0) {
            $this->error('当前订单状态不允许该操作');
        }

        $material = Db::table('cb_crm_copyright_opus_materials')->where('order_id', $row->id)->find();

        $params = $this->request->post();
        $res = false;
        Db::startTrans();
        try {
            if ($step == 1) {
                $params['opusName']?:$this->error('请输入作品名称');
                if (($params['opusType'] == 'A' || $params['opusType'] == 'M') && !(int)$params['opusTypeDesc']) {
                    $this->error('请输入作品字数，且只能输入数字');
                }
                if ($params['opusType'] == 'L' && !$params['opusTypeDesc']) {
                    $this->error('请输入作品类别说明');
                }
                if ($params['opusType'] == 'I' && (!$params['opusTypeDesc'] || $params['opusTypeDesc'] == '0h0m0s' || $params['opusTypeDesc'] == 'h0m0s')) {
                    $this->error('请输入作品时长');
                }
                if ($params['publishStatus'] == 1 && (!$params['appearDate'] || !$params['appearCity'])) {
                    $this->error('已发表的作品请选择首次发表时间和发表地点');
                }
                $params['completeDate']?:$this->error('请输入创作完成日期');
                $params['completeCity']?:$this->error('请选择创作完成地点');
                $params['opusSamples']?:$this->error('请上传作品样本');
                $params['user_id'] = $this->auth->id;
                $params['order_id'] = $row->id;
                $params['isDraft'] = 1;
                $params['create_time'] = date('Y-m-d H:i:s');
                if ($material) {
                    // 存在则更新
                    $params['update_time'] = date('Y-m-d H:i:s');
                    $res = Db::table('cb_crm_copyright_opus_materials')->where('id', $material['id'])->update($params);
                } else {
                    // 新增
                    $res = Db::table('cb_crm_copyright_opus_materials')->insert($params);
                }
            } elseif ($step == 2) {
                $owners = json_decode(htmlspecialchars_decode(html_entity_decode($params['owners'])), true);
                $author =json_decode(htmlspecialchars_decode(html_entity_decode($params['author'])), true);

                if ($params['obtainType'] == 'INHERIT' && (!$params['heirRelationshipProve'] || !$params['deathProve'] || !$params['validWill'])) {
                    $this->error('继承时,相关证明文件都必须上传');
                }
                if ($params['obtainType'] == 'REORGANIZATION' && (!$params['enterpriseChangeProve'] && !$params['contract'] && !$params['creditorsMeetingResolution'] && !$params['liquidationNotice'])) {
                    $this->error('承受时,相关证明文件至少上传一个');
                }
                if ($params['obtainType'] == 'TRANSFER' && (!$params['transferContract'] || !$params['rightObtainDate'])) {
                    $this->error('转让时,转让合同和权利取得日期必须上传/填写');
                }
                if ($params['rightOwnType'] == 'COT' && count($author) < 2) {
                    $this->error('合作作品作者至少填写2项，请继续添加作者');
                }
                if (count(explode(',',$params['rightScopePart'])) == 0) {
                    $this->error('请选择权利拥有项');
                }
                $params['jointWorksStatement']?:$this->error('请上传作品创作声明');
                $num = 1;
                $ownersData = [];
                foreach ($owners as $key => $owner) {
                    !$owner['cityName'] ? $this->error("请选择第{$num}个著作权人地区信息") : true;
                    !$owner['peopleType'] ? $this->error("请选择第{$num}个著作权人身份类别") : true;
                    !$owner['name'] ? $this->error("请填写第{$num}个著作权人姓名或名称") : true;
                    !$owner['idType'] ? $this->error("请选择第{$num}个著作权人证件类型") : true;
                    !$owner['idNumber'] ? $this->error("请填写第{$num}个著作权人证件号码") : true;

                    if (($owner['idType'] == '1' || $owner['idType'] == '21') && (!$owner['cardFront'] || !$owner['cardBack'])) {
                        $this->error("请完整上传第{$num}个著作权人证件材料附件");
                    } elseif ($owner['idType'] != '1' && $owner['idType'] != '21' && !$owner['cardFront']) {
                        $this->error("请上传第{$num}个著作权人证件材料附件");
                    }
                    !$owner['phone'] ? $this->error("请填写第{$num}个著作权人手机号码") : true;
                    !$owner['contactPerson'] ? $this->error("请填写第{$num}个著作权人联系人姓名") : true;
                    !$owner['email'] ? $this->error("请填写第{$num}个著作权人电子邮箱地址") : true;
                    $num++;
                    array_push($ownersData, $owner);
                }
                $params['owners'] = json_encode($ownersData, 256);
                $num = 1;
                $authorData = [];
                foreach ($author as $item) {
                    !$item['name'] ? $this->error("请填写第{$num}个作者姓名/名称") : true;
                    array_push($authorData, $item);
                }
                $params['author'] = json_encode($authorData, 256);
                $params['update_time'] = date('Y-m-d H:i:s');
                $params['isDraft'] = 0;
                $res = Db::table('cb_crm_copyright_opus_materials')->where('id', $material['id'])->update($params);
                $row->save(['status' => 1]);
            }
            Db::commit();
        } catch (Exception $e) {
            Db::rollback();
            $this->error($e->getMessage());
        }
        if ($res !== false) {
            $this->success('材料提交成功');
        } else {
            $this->error('材料提交失败');
        }
    }

    /*
     * 软著提交材料数据字典
     * */
    public function softwareMaterialData()
    {
        $mdoel = new OrderModel();

        $idCardType = $mdoel->getIdCardType();
        $rightObtainMode = $mdoel->getRightObtainMode();
        $inheritanceType = $mdoel->getInheritanceType();
        $rightScopePart = $mdoel->getRightScopePart();
        $softwareDescription = $mdoel->getSoftwareDescription();
        $developMode = $mdoel->getDevelopMode();
        $programLanguage = $mdoel->getProgramLanguage();
        $specialtyTags = $mdoel->getSpecialtyTags();

        $this->success('返回成功', compact('idCardType', 'rightObtainMode', 'inheritanceType', 'softwareDescription', 'developMode', 'rightScopePart', 'programLanguage', 'specialtyTags'));
    }

    /*
     * 作品著作权提交材料数据字典
     * */
    public function opusMaterialData()
    {
        $mdoel = new OrderModel();
        $opusType = $mdoel->getOpusType();
        $opusInditeType = $mdoel->getOpusInditeType();
        $obtainType = $mdoel->getObtainType();
        $rightOwnType = $mdoel->getRightOwnType();
        $idCardType = $mdoel->getIdCardType();
        $rightScopePart = $mdoel->getOpusRightScopePart();

        $this->success('返回成功', compact('opusType', 'opusInditeType', 'obtainType', 'rightOwnType', 'idCardType', 'rightScopePart'));
    }

    public function createOrder()
    {
        $params = [];
        $copyright_id = $this->request->post('copyright_id') ?: $this->error('产品参数错误');
        $spec_id = $this->request->post('spec_id') ?: $this->error('认证时间参数错误');

        $product = (new ProductModel())->where('id', $copyright_id)->where('status', 1)->find();

        if (!$product) {
            $this->error('服务不存在');
        }

        $spec = Db::table('cb_crm_copyright_sepc')
            ->where('delete_time', null)
            ->where('id', $spec_id)
            ->field('name,price')
            ->find();
        if (!$spec) {
            $this->error('服务规格不存在');
        }
        $params['wid'] = $this->wid;
        $params['user_id'] = $this->auth->id;
        $params['order_no'] = date('YmdHis') . rand(10000, 99999);
        $params['copyright_id'] = $copyright_id;
        $params['spec_id'] = $spec_id;
        $params['name'] = $product->getAttr('name');
        $params['spec_attr'] = $spec['name'];
        $params['amount'] = $spec['price'];
        $params['pay_status'] = 0;
        $params['create_time'] = date('Y-m-d H:i:s');
        $params['update_time'] = date('Y-m-d H:i:s');

        $order_id = (new OrderModel())->insertGetId($params);

        $data = [
            'order_id' => $order_id,
            'order_money' => $params['amount'],
            'balance' => $this->auth->balance
        ];

        $this->success('返回成功', $data);

    }

    public function payOrder()
    {
        $order_id = $this->request->post('order_id', 0, 'intval') ?: $this->error('订单不存在');
        $platform = $this->request->post('platform', 'wechat');
        $pay_type = $this->request->post('pay_type', 'wechat');
        $order = (new OrderModel())->where('id', $order_id)->where('user_id', $this->auth->id)->find();
        if (!$order || $order->status != 0) {
            $this->error('订单不存在', $order);
        }

        $result = false;
        Db::startTrans();
        try {

            if ($pay_type == 'wechat') {
                $result = $this->payWechat($order->amount, $order->order_no, $order->getAttr('name'), $platform);
                if (!$result) {
                    throw new Exception('订单支付异常');
                }
            } elseif ($pay_type == 'balance') {
                $result = $this->payBalance($order->amount, $order->order_no);
                if (!$result) {
                    throw new Exception('余额支付异常');
                }
            }
            Db::commit();
        } catch (Exception $e) {
            Db::rollback();
            $this->error($e->getMessage());
        }
        if ($result) {
            $this->success('支付成功', $result);
        } else {
            $this->error('订单支付异常');
        }
    }

    /**
     * @title 微信支付
     * @param float $amount 订单金额
     * @param string $order_no 订单号
     * @param string $goods_name 商品名称
     * @param string $platform 支付平台 wechat 公众号   mpWechat 微信小程序
     * */
    protected function payWechat($amount, $order_no, $goods_name, $platform)
    {
        $config = (new SettingModel())->getConfigGroup('wechat', $this->wid);
        $payConfig = [
            // 必要配置
            'app_id' => $config['app_id'],
            'mch_id' => $config['mch_id'],
            'key' => $config['api_key'],   // API v2 密钥 (注意: 是v2密钥 是v2密钥 是v2密钥)
            'notify_url' => $this->request->domain() . '/home/api.notify/copyright_order/wid/' . $this->wid . '/platform/' . $platform,     // 你也可以在下单时单独设置来想覆盖它
        ];
        if ($platform == 'mpWechat') {
            $payConfig['app_id'] = $config['miniapp_id'];
            $payConfig['secret'] = $config['miniapp_secret'];
        }

        $openid = Db::table('cb_crm_user_third')->where('user_id', $this->auth->id)->where('platform', $platform)->value('openid');

        $isWechat = strpos($this->request->server('HTTP_USER_AGENT'), 'MicroMessenger') !== false;
        $isMiniProgram = strpos($this->request->server('HTTP_USER_AGENT'), 'MiniProgram') !== false;
        if ($isMiniProgram || $isWechat) {
            $trade_type = 'JSAPI';
        } else {
            $trade_type = 'MWEB';
        }

        $app = Factory::payment($payConfig);
        $payData = null;
        try {
            $result = $app->order->unify([
                'body' => '购买' . $goods_name,
                'openid' => $openid,
                'out_trade_no' => $order_no,
                'total_fee' => $amount * 100, // 单位：分
                'trade_type' => $trade_type, // 请对应换成你的支付方式对应的值类型
            ]);
            if (isset($result['return_code']) && $result['return_code'] === 'FAIL') {
                throw new Exception($result['return_msg']);
            }
            if (isset($result['result_code']) && $result['result_code'] === 'FAIL') {
                throw new Exception($result['err_code_des']);
            }
            if ($trade_type == 'JSAPI') {
                $jssdk = $app->jssdk;
                $payData = $jssdk->bridgeConfig($result['prepay_id']);
            };

        } catch (Exception $e) {
            $this->error($e->getMessage());
        } catch (\EasyWeChat\Kernel\Exceptions\InvalidArgumentException $e) {
            $this->error($e->getMessage());
        } catch (\EasyWeChat\Kernel\Exceptions\InvalidConfigException $e) {
            $this->error($e->getMessage());
        } catch (\GuzzleHttp\Exception\GuzzleException $e) {
            $this->error($e->getMessage());
        }

        return $payData;
    }

    /**
     * @title 余额支付
     * @param float $amount 订单金额
     * @param string $order_no 订单号
     * */
    protected function payBalance($amount, $order_no)
    {
        if ($this->auth->balance < $amount) {
            $this->error('可用余额不足');
        }
        $order = (new OrderModel())->where('order_no', $order_no)->find();
        if (!$order || $order->pay_status == 1) {
            $this->error('待支付订单不存在');
        }

        // 修改订单支付状态
        $order->pay_status = 1;
        $order->pay_time = date('Y-m-d H:i:s');

        $res = $order->save();


        $userInfo = (new MemberModel())->where('id', $this->auth->id)->find();
        // 扣除用户余额
        $userInfo->balance = $userInfo->balance - $amount;

        $userInfo->save();

        if ($res) {
            // 发送短信通知
            $smsServices = app()->make(SmsServices::class);
            $mobile = (new SettingModel())->getAcceptPhone($this->wid);
            $smsServices->send($mobile,['type'=>'版权登记','order_no'=>$order_no],'后台新订单',$this->wid);
            return true;
        } else {
            return false;
        }
    }

}
