<?php


namespace app\cbcrm\controller\api;


use app\cbcrm\model\config\DictionaryModel;
use app\cbcrm\model\config\SettingModel;
use app\cbcrm\model\logo\CaseModel;
use app\cbcrm\model\logo\LogoViModel;
use app\cbcrm\model\logo\OrderDetailModel;
use app\cbcrm\model\logo\OrderModel;
use app\cbcrm\model\logo\PackageModel;
use app\cbcrm\model\member\MemberModel;
use app\cbcrm\services\SmsServices;
use EasyWeChat\Factory;
use think\App;
use think\Exception;
use think\facade\Db;

class Logo extends Base
{
    public function __construct(App $app)
    {
        parent::__construct($app);
        $this->auth = $this->auth();
    }

    public function product()
    {
        $model = new PackageModel();

        $list = $model->where('wid',$this->wid)
            ->where('status',1)
            ->order('sort desc, id asc')
            ->select();
        foreach ($list as $key=>$item) {
            $list[$key]['feature'] = explode(',',$item['feature']);
            $list[$key]['standard'] = explode(',',$item['standard']);
        }
        $this->success('返回成功',$list);
    }

    public function detail()
    {
        $id = $this->request->get('id', 0, 'intval') ?: $this->error('参数错误');

        $row = (new PackageModel())
            ->where('id', $id)
            ->where('status', 1)
            ->where('wid', $this->wid)
            ->find();
        if (!$row) {
            $this->error('未找到产品');
        }

        //  配套VI
        $viList = (new LogoViModel())->where('wid', $this->wid)
            ->where('status', 1)
            ->order('sort desc, id asc')
            ->select();

        $viData = [
            'basic' => [],
            'company' => [],
            'store' => []
        ];
        // 类型:0=基本VI,1=企业VI,2=店铺VI
        foreach ($viList as $vi) {
            switch ($vi['type']) {
                case 0:
                    $viData['basic'][] = $vi;
                    break;
                case 1:
                    $viData['company'][] = $vi;
                    break;
                case 2:
                    $viData['store'][] = $vi;
                    break;
            }
        }

        // 行业
        $categoryData  = (new DictionaryModel())->getTypeValues('87',$this->wid);
        // 附加服务
        $otherService = Db::table('cb_crm_logo_other_service')->where('wid',$this->wid)->order('sort desc,id asc')->select();

        $this->success('返回成功', [
            'row' => $row,
            'viData' => $viData,
            'categoryData'=>$categoryData,
            'otherService'=>$otherService
        ]);

    }

    public function getCaseList()
    {
       $category_id =  $this->request->get('category_id', 0, 'intval') ?: $this->error('参数错误');
       $caseList = (new CaseModel())
           ->where('category_id',$category_id)
           ->where('wid', $this->wid)
           ->where('status', 1)
           ->order('sort desc, id asc')
           ->select();
        foreach ($caseList as $key=>$item) {
            $caseList[$key]['logo'] = cdnurl($item['logo'],true);
        }

       $this->success('返回成功',$caseList);
    }

    public function createOrder()
    {
        $package_id = $this->request->post('package_id',0,'intval')?:$this->error('参数错误');

        $logo_name = $this->request->post('logo_name');
        $category_id = $this->request->post('category_id');
        $introduce = $this->request->post('introduce');
        $idea = $this->request->post('idea');
        $other_service = $this->request->post('other_service');
        $basicVi = $this->request->post('basicVi');
        $companyVi = $this->request->post('companyVi');
        $storeVi = $this->request->post('storeVi');
        $defaultVi = $this->request->post('defaultVi');
        $case_files = $this->request->post('case_files');
        $idea_files = $this->request->post('idea_files');

        $row = (new PackageModel())
            ->where('id', $package_id)
            ->where('status', 1)
            ->where('wid', $this->wid)
            ->find();
        if (!$row) {
            $this->error('未找到产品');
        }

        // 计算订单金额
        $orderNo = date('YmdHis') . rand(10000, 99999);
        $orderMoney = $row->price;
        // 加配套VI价格
        if ($basicVi || $companyVi || $storeVi || $defaultVi) {
            $orderMoney += $row->vi_price;
        }
        // 如果勾选的配置VI超过$row->vi_num项，则计算加项的金额
        $viList = [];
        if ($basicVi) {
            array_push($viList,$basicVi);
        }
        if ($companyVi) {
            array_push($viList,$companyVi);
        }
        if ($storeVi) {
            array_push($viList,$storeVi);
        }
        if ($defaultVi) {
            array_push($viList,$defaultVi);
        }
        if ($viList) {
            $viCount = count(explode(',',implode(',',$viList)));
            if ($viCount > $row->vi_num) {
                $orderMoney += $row->unit_vi_price * ($viCount - $row->vi_num);
            }
        }
        // 计算其他服务
        $otherServiceData = [];
        if (!empty($other_service)) {
            $otherServiceData = Db::table('cb_crm_logo_other_service')->where('wid',$this->wid)->where('id','IN',$other_service)->field('title as name,price')->select();
            foreach ($otherServiceData as $item) {
                $orderMoney += $item['price'];
            }
        }

        $resutl = false;
        Db::startTrans();
        try {
            // 创建订单
            $orderData = [
                'wid'=>$this->wid,
                'user_id'=>$this->auth->id,
                'order_no'=>$orderNo,
                'price'=>$orderMoney,
                'pay_status'=>0,
                'status'=>0,
                'package_id'=>$row->id,
                'package_info'=>json_encode($row,JSON_UNESCAPED_UNICODE),
                'create_time'=>date('Y-m-d H:i:s'),
                'update_time'=>date('Y-m-d H:i:s')
            ];
            $order_id = (new OrderModel())->createOrder($orderData);
            // 订单详情
            $orderDetailData = [
                'wid'=>$this->wid,
                'order_id'=>$order_id,
                'logo_name'=>$logo_name,
                'category_id'=>$category_id,
                'case_files'=>$case_files,
                'introduce'=>$introduce,
                'idea'=>$idea,
                'idea_files'=>$idea_files,
                'vi'=>trim(implode(',',[$basicVi,$companyVi,$storeVi,$defaultVi]),','),
                'other_service'=>json_encode($otherServiceData,JSON_UNESCAPED_UNICODE)
            ];
            (new OrderDetailModel())->createOrderDetail($orderDetailData);
            $resutl = [
                'order_id'=>$order_id,
                'order_money'=>$orderMoney,
                'balance'=>$this->auth->balance
            ];
            Db::commit();
        } catch (Exception $e) {
            Db::rollback();
            $this->error($e->getMessage());
        }
        $this->success('订单创建成功',$resutl);
    }


    public function payOrder()
    {
        $order_id = $this->request->post('order_id',0,'intval')?:$this->error('订单不存在');
        $platform = $this->request->post('platform', 'wechat');
        $pay_type = $this->request->post('pay_type', 'wechat');
        $order = (new OrderModel())->where('id',$order_id)->where('user_id',$this->auth->id)->find();
        if (!$order || $order->pay_status != 0) {
            $this->error('订单不存在');
        }

        $packageInfo = json_decode($order->package_info,true);

        $result = false;
        Db::startTrans();
        try {

            if ($pay_type == 'wechat') {
                $result = $this->payWechat($order->price, $order->order_no,'LOGO设计 - '.$packageInfo['title'], $platform);
                if (!$result) {
                    throw new Exception('订单支付异常');
                }
            } elseif ($pay_type == 'balance') {
                $result = $this->payBalance($order->price, $order->order_no);
                if (!$result) {
                    throw new Exception('余额支付异常');
                }
            }
            Db::commit();
        } catch (Exception $e) {
            Db::rollback();
            $this->error($e->getMessage());
        }
        if ($result) {
            $this->success('支付成功', $result);
        } else {
            $this->error('订单支付异常');
        }
    }
    /**
     * @title 微信支付
     * @param float $amount 订单金额
     * @param string $order_no 订单号
     * @param string $goods_name 商品名称
     * @param string $platform 支付平台 wechat 公众号   mpWechat 微信小程序
     * */
    protected function payWechat($amount, $order_no, $goods_name, $platform){
        $config = (new SettingModel())->getConfigGroup('wechat', $this->wid);
        $payConfig = [
            // 必要配置
            'app_id' => $config['app_id'],
            'mch_id' => $config['mch_id'],
            'key' => $config['api_key'],   // API v2 密钥 (注意: 是v2密钥 是v2密钥 是v2密钥)
            'notify_url' => $this->request->domain() . '/home/api.notify/logo_order/wid/' . $this->wid . '/platform/' . $platform,     // 你也可以在下单时单独设置来想覆盖它
        ];
        if ($platform == 'mpWechat') {
            $payConfig['app_id'] = $config['miniapp_id'];
            $payConfig['secret'] = $config['miniapp_secret'];
        }

        $openid = Db::table('cb_crm_user_third')->where('user_id', $this->auth->id)->where('platform', $platform)->value('openid');

        $isWechat = strpos($this->request->server('HTTP_USER_AGENT'), 'MicroMessenger') !== false;
        $isMiniProgram = strpos($this->request->server('HTTP_USER_AGENT'), 'MiniProgram') !== false;
        if ($isMiniProgram || $isWechat) {
            $trade_type = 'JSAPI';
        } else {
            $trade_type = 'MWEB';
        }

        $app = Factory::payment($payConfig);
        $payData = null;
        try {
            $result = $app->order->unify([
                'body' => '购买' . $goods_name,
                'openid' => $openid,
                'out_trade_no' => $order_no,
                'total_fee' => $amount * 100, // 单位：分
                'trade_type' => $trade_type, // 请对应换成你的支付方式对应的值类型
            ]);
            if (isset($result['return_code']) && $result['return_code'] === 'FAIL') {
                throw new Exception($result['return_msg']);
            }
            if (isset($result['result_code']) && $result['result_code'] === 'FAIL') {
                throw new Exception($result['err_code_des']);
            }
            if ($trade_type == 'JSAPI') {
                $jssdk = $app->jssdk;
                $payData = $jssdk->bridgeConfig($result['prepay_id']);
            };

        } catch (Exception $e) {
            $this->error($e->getMessage());
        } catch (\EasyWeChat\Kernel\Exceptions\InvalidArgumentException $e) {
            $this->error($e->getMessage());
        } catch (\EasyWeChat\Kernel\Exceptions\InvalidConfigException $e) {
            $this->error($e->getMessage());
        } catch (\GuzzleHttp\Exception\GuzzleException $e) {
            $this->error($e->getMessage());
        }

        return $payData;
    }

    /**
     * @title 余额支付
     * @param float $amount 订单金额
     * @param string $order_no 订单号
     * */
    protected function payBalance($amount, $order_no)
    {
        if ($this->auth->balance < $amount) {
            $this->error('可用余额不足');
        }
        $order = (new OrderModel())->where('order_no', $order_no)->find();
        if (!$order || $order->pay_status == 1) {
            $this->error('待支付订单不存在');
        }

        // 修改订单支付状态
        $order->pay_status = 1;
        $order->pay_time = date('Y-m-d H:i:s');

        $res = $order->save();


        $userInfo = (new MemberModel())->where('id', $this->auth->id)->find();
        // 扣除用户余额
        $userInfo->balance = $userInfo->balance - $amount;

        $userInfo->save();

        if ($res) {
            // 发送短信通知
            $smsServices = app()->make(SmsServices::class);
            $mobile = (new SettingModel())->getAcceptPhone($this->wid);
            $smsServices->send($mobile,['type'=>'LOGO设计','order_no'=>$order_no],'后台新订单',$this->wid);
            return true;
        } else {
            return false;
        }
    }


}
