<?php


namespace app\cbcrm\controller\api;


use app\cbcrm\model\config\DeliveryModel;
use app\cbcrm\model\config\SettingModel;
use app\cbcrm\model\goods\GoodsModel;
use app\cbcrm\model\goods\OrderDetailModel;
use app\cbcrm\model\goods\OrderModel;
use app\cbcrm\model\market\CouponModel;
use app\cbcrm\model\member\GradeModel;
use app\cbcrm\model\member\MemberModel;
use app\cbcrm\services\SmsServices;
use EasyWeChat\Factory;
use think\App;
use think\Exception;
use think\facade\Db;
use think\facade\Log;

class Order extends Base
{
    public function __construct(App $app)
    {
        parent::__construct($app);
        $this->auth = $this->auth();
    }

    /*
     * 创建预订单
     * */
    public function orderBefore()
    {
        $goods_id = $this->request->get('goods_id');
        $buy_num = $this->request->get('buy_num', 1);
        $spec_id = $this->request->get('spec_id');// 获取商品信息
        $address_id = $this->request->get('address_id', 0);
        $goodsInfo = (new GoodsModel())->getDetail($goods_id, $this->wid);
        if (!$goodsInfo || $goodsInfo->status != 1) {
            $this->error('购买的商品不存在');
        }
        $goodsSpec = Db::table('cb_crm_goods_spec')->where('goods_id', $goods_id)->where('id', $spec_id)->find();
        if (!$goodsSpec) {
            $this->error('购买的商品规格不存在');
        }
        if ($goodsSpec['stock_num'] < $buy_num) {
            $this->error('购买的商品规格数量不足');
        }

        if ($goodsSpec['image']) {
            $goodsSpec['image'] = cdnurl($goodsSpec['image']);
        } else {
            $goodsSpec['image'] = $goodsInfo->goods_images[0]['preview_url'];
        }

        $goodsInfo->selectSpec = $goodsSpec;
        // 商品总价
        $total_price = $goodsSpec['goods_price'] * $buy_num;

        // 可用优惠券
        $today = date('Y-m-d');
        $whereStr = "`user_id` = {$this->auth->id} AND `is_expire` = 0  AND `is_use` = 0 
        AND (`apply_range` = 1 OR FIND_IN_SET('{$goodsInfo->id}',`apply_range_config`)) 
        AND (`expire_type` = 1 OR (`start_time` <= '{$today}' AND `end_time` >= '{$today}'))";


        $couponList = Db::table('cb_crm_user_coupon')->where($whereStr)->select()->toArray();

        foreach ($couponList as $key => $item) {
            $couponList[$key]['shortname'] = $item['coupon_name'];
            if ($item['coupon_type'] == 1) {
                $couponList[$key]['shortname'] = '满' . $item['min_price'] . '减' . $item['reduce_price'];
            } elseif ($item['coupon_type'] == 2) {
                $couponList[$key]['shortname'] = '满' . $item['min_price'] . '元' . $item['discount'] . '折';
            }
        }
        $goodsInfo->couponList = $couponList;

        // 会员抵扣
        // 会员等级抵扣 是否开启会员折扣(1开启 0关闭)
        $is_user_grade = 0; // 是否存在会员等级折扣
        $grade_ratio = 0; // 会员折扣比例
        if ($goodsInfo->is_enable_grade == 1) {
            // 会员折扣设置(0默认等级折扣 1单独设置折扣)
            if ($goodsInfo->is_alone_grade == 0) {
                // 当前下单用户会员等级可折扣
                $grade_ratio = (new GradeModel())->where('wid', $this->wid)->where('id', $this->auth->grade_id)->value('discount');
            } elseif ($goodsInfo->is_alone_grade == 1) {
                if (is_array($goodsInfo->alone_grade_equity)) {
                    $alone_grade_equity = $goodsInfo->alone_grade_equity;
                } else {
                    $alone_grade_equity = json_decode($goodsInfo->alone_grade_equity, true);
                }
                $key = 'grade_id:' . $this->auth->grade_id;
                $grade_ratio = isset($alone_grade_equity[$key]) ? $alone_grade_equity[$key] : 0;
            }
            if ($grade_ratio > 0) {
                $is_user_grade = 1;
            }
        }
        $goodsInfo->is_user_grade = $is_user_grade;
        $goodsInfo->grade_ratio = $grade_ratio;

        // 积分抵扣
        // 会员积分抵扣 优先判断系统是否开启积分抵扣，再判断商品单独设置的积分抵扣
        $allow_use_score = false;
        $scoreConfig = (new SettingModel())->getConfigGroup('score', $this->wid);
        if ($scoreConfig['is_shopping_discount'] == 1 && $goodsInfo->is_points_discount == 1) {
            $allow_use_score = true;
        }
        $scoreData = [
            'allow_use_score' => $allow_use_score,
            'config' => $scoreConfig,
            'userScoreBalance' => $this->auth->score
        ];

        // 收货地址
        $addressWhere = [
            ['user_address.user_id', '=', $this->auth->id]
        ];
        if ($address_id) {
            $addressWhere[] = ['user_address.id', '=', $address_id];
        }
        $address = Db::table('cb_crm_user_address')
            ->alias('user_address')
            ->join(['cb_crm_area' => 'province'], 'user_address.province_id = province.id', 'left')
            ->join(['cb_crm_area' => 'city'], 'user_address.city_id = city.id', 'left')
            ->join(['cb_crm_area' => 'region'], 'user_address.region_id = region.id', 'left')
            ->where($addressWhere)
            ->order('user_address.create_time', 'desc')
            ->field('user_address.*,province.name as province,city.name as city,region.name as region')
            ->find();
        $userInfo = $this->auth();

        $this->success('返回成功', compact('goodsInfo', 'scoreData', 'address', 'userInfo'));
    }

    /*
     * 创建订单
     * */
    public function createOrder()
    {
        $goods_id = $this->request->post('goods_id');
        $buy_num = $this->request->post('buy_num', 1);
        $spec_id = $this->request->post('spec_id');
        $coupon_id = $this->request->post('coupon_id');
        $address_id = $this->request->post('address_id');
        $buyer_remark = $this->request->post('buyer_remark');
        $platform = $this->request->post('platform', 'wechat');
        $score = $this->request->post('score', 0);// 使用积分
        $pay_type = $this->request->post('pay_type', 'wechat'); // 支付方式

        // 获取商品信息
        $goodsInfo = (new GoodsModel())->getDetail($goods_id, $this->wid);
        if (!$goodsInfo || $goodsInfo->status != 1) {
            $this->error('购买的商品不存在');
        }
        $goodsSpec = Db::table('cb_crm_goods_spec')->where('goods_id', $goods_id)->where('id', $spec_id)->find();
        if (!$goodsSpec) {
            $this->error('购买的商品规格不存在');
        }
        if ($goodsSpec['stock_num'] < $buy_num) {
            $this->error('购买的商品规格库存不足');
        }

        $addressInfo = Db::table('cb_crm_user_address')->where('id', $address_id)->where('user_id', $this->auth()->id)->find();
        if (!$addressInfo) {
            $this->error('收货地址不存在');
        }

        // 商品总价
        $total_price = $goodsSpec['goods_price'] * $buy_num;
        $order_price = $total_price;
        // 订单价格 = (商品价格 * 商品数量）- 优惠券优惠金额 - 会员折扣 - 积分抵扣金额

        // 优惠劵抵扣
        $coupon_money = 0;
        if ($coupon_id) {
            $couponInfo = (new CouponModel())->getDetail($coupon_id, $this->wid);
            // 验证优惠券是否有效
            if ($couponInfo) {
                $userCoupon = Db::table('cb_crm_user_coupon')->where('user_id', $this->auth->id)->where('coupon_id', $coupon_id)->find();
                if (!$userCoupon) {
                    $this->error('未领取到该优惠券');
                }
                // 有效期 expire_type： 1 = 领取后生效  expire_type： 2 = 固定时间
                if ($userCoupon['expire_type'] == 1 && strtotime("+{$userCoupon['expire_day']} days", strtotime($userCoupon['create_time'])) < time()) {
                    $this->error('所使用的优惠券已到期');
                }
                if ($userCoupon['expire_type'] == 2 && (strtotime($userCoupon['start_time']) > time() || strtotime($userCoupon['end_time']) < time())) {
                    $this->error('所使用的优惠券不在有效期内');
                }
                // 适用范围
                if ($userCoupon['apply_range'] == 2) {
                    $applyGoodsIdsArr = explode(',', $userCoupon['apply_range_config']);
                    if (!in_array($goods_id, $applyGoodsIdsArr)) {
                        $this->error('该商品不在优惠劵使用范围内');
                    }
                }
                // 优惠券类型:1=满减券,2=折扣券
                if ($userCoupon['min_price'] > 0 && $total_price < $userCoupon['min_price']) {
                    $this->error('商品总价未达到优惠劵最低消费金额');
                }
                // 开始抵扣优惠劵
                if ($userCoupon['coupon_type'] == 1) {
                    // 满减
                    $order_price = $order_price - $userCoupon['reduce_price'];
                    $coupon_money = $userCoupon['reduce_price'];
                } else {
                    // 折扣券
                    $order_price = $order_price * ($userCoupon['discount'] / 10);
                    $coupon_money = $total_price - $order_price;
                }

            } else {
                $this->error('所选的优惠券无效');
            }
        }
        // 会员等级抵扣 是否开启会员折扣(1开启 0关闭)
        $is_user_grade = 0; // 是否存在会员等级折扣
        $grade_ratio = 0; // 会员折扣比例
        $grade_goods_price = 0; // 会员折扣的商品单价
        $grade_total_money = 0; // 会员折扣的总额差
        if ($goodsInfo->is_enable_grade == 1) {
            // 会员折扣设置(0默认等级折扣 1单独设置折扣)
            if ($goodsInfo->is_alone_grade == 0) {
                // 当前下单用户会员等级可折扣
                $grade_ratio = (new GradeModel())->where('wid', $this->wid)->where('id', $this->auth->grade_id)->value('discount');
            } elseif ($goodsInfo->is_alone_grade == 1) {
                if (is_array($goodsInfo->alone_grade_equity)) {
                    $alone_grade_equity = $goodsInfo->alone_grade_equity;
                } else {
                    $alone_grade_equity = json_decode($goodsInfo->alone_grade_equity, true);
                }

                $key = 'grade_id:' . $this->auth->grade_id;
                $grade_ratio = isset($alone_grade_equity[$key]) ? $alone_grade_equity[$key] : 0;

            }
            if ($grade_ratio > 0) {
                $is_user_grade = 1;
                $grade_goods_price = $goodsSpec['goods_price'] * ($grade_ratio / 10);
                $grade_total_money = number_format(($goodsSpec['goods_price'] * $buy_num) - (($goodsSpec['goods_price'] * $buy_num) * ($grade_ratio / 10)), 2);
                $order_price = $order_price - $grade_total_money;
            }
        }
        // 会员积分抵扣 优先判断系统是否开启积分抵扣，再判断商品单独设置的积分抵扣
        $score_money = 0;
        $scoreConfig = (new SettingModel())->getConfigGroup('score', $this->wid);
        if ($scoreConfig['is_shopping_discount'] == 1 && $goodsInfo->is_points_discount == 1 && $score) {
            if ($total_price < $scoreConfig['full_order_price']) {
                $this->error('当前订单金额未满足积分使用条件');
            }
            // 可抵扣金额
            $scoreDiscount = $score_money = $score * $scoreConfig['discount_ratio'];
            $maxDiscount = $total_price * ($scoreConfig['max_money_ratio'] / 100);
            if ($scoreDiscount > $maxDiscount) {
                $this->error('抵扣金额超出允许范围内');
            };
            // 抵扣金额
            $order_price = $order_price - $scoreDiscount;
        }

        // 运费
        $deliveryMoney = 0;

        // 满额包邮
        $fullFreeConfig = (new SettingModel())->getConfigGroup('full_free', $this->wid);
        $isFreeDelivery = false; // 是否包邮
        if ($fullFreeConfig['is_open'] == 1 && $goodsInfo->type == 1) {
            $excludedGoodsIds = json_decode($fullFreeConfig['excludedGoodsIds'], true);
            $excludedRegions = json_decode($fullFreeConfig['excludedRegions'], true);

            if (!in_array($goods_id, $excludedGoodsIds) && !in_array($addressInfo['city'], $excludedRegions['cityIds']) && $total_price > $fullFreeConfig['money']) {
                $isFreeDelivery = true;
            }
        }
        if ($goodsInfo->type == 1 && $goodsInfo->delivery_id && !$isFreeDelivery) {
            $deliveryInfo = (new DeliveryModel())->with(['rule'])->where('id', $goodsInfo->delivery_id)->find();
            $deliveryRules = $deliveryInfo->rule;

            // 可配送区域
            $deliveryRegion = [];
            $ruleIndex = false;
            foreach ($deliveryRules as $key => $rule) {
                $region = json_decode($rule->region, true);
                $deliveryRegion = array_merge($deliveryRegion, $region);
                if (in_array($addressInfo['province_id'], $region) || in_array($addressInfo['city_id'], $region)) {
                    $ruleIndex = $key;
                }
            }
            if ($ruleIndex === false) {
                $this->error('当前收货地址不在配送范围');
            }
            // 计费方式:1=按件,2=按重
            if ($deliveryInfo->method == 1) {
                if ($buy_num <= $deliveryRules[$ruleIndex]['first_unit']) {
                    $deliveryMoney = $deliveryRules[$ruleIndex]['first_fee'];
                } else {
                    // 首件价格 +（（总数 - 首件数量） / 续件数量） * 续件价格
                    if ($deliveryRules[$ruleIndex]['additional'] > 0) {
                        $deliveryMoney = $deliveryRules[$ruleIndex]['first_fee'] + ($buy_num - $deliveryRules[$ruleIndex]['first_unit']) / $deliveryRules[$ruleIndex]['additional'] * $deliveryRules[$ruleIndex]['additional_fee'];
                    } else {
                        $deliveryMoney = $deliveryRules[$ruleIndex]['first_fee'];
                    }
                }
            } else {
                if (($goodsSpec->goods_weight * $buy_num) <= $deliveryRules[$ruleIndex]['first_unit']) {
                    $deliveryMoney = $deliveryRules[$ruleIndex]['first_fee'];
                } else {
                    // 首重价格 +（（总商品重量 - 首重） / 续重） * 续重价格
                    if ($deliveryRules[$ruleIndex]['additional'] > 0) {
                        $deliveryMoney = $deliveryRules[$ruleIndex]['first_fee'] + (($goodsSpec->goods_weight * $buy_num) - $deliveryRules[$ruleIndex]['first_unit']) / $deliveryRules[$ruleIndex]['additional'] * $deliveryRules[$ruleIndex]['additional_fee'];
                    } else {
                        $deliveryMoney = $deliveryRules[$ruleIndex]['first_fee'];
                    }
                }
            }
        }


        // 积分赠送 订单付款金额(100.00元) * 积分赠送比例(100%) = 实际赠送的积分(100积分) 订单的运费不参与积分赠送
        $scoreGift = 0;
        if ($scoreConfig['is_shopping_gift'] == 1 && $goodsInfo->is_points_gift == 1) {
            $scoreGift = $order_price * ($scoreConfig['gift_ratio'] / 100);
        }

        $order_no = date('YmdHis') . rand(10000, 99999);
        // 实际需支付金额 = 订单价格加运费
        $pay_price = $order_price + $deliveryMoney;
        if ($pay_price < 0.01) {
            $this->error('订单价格异常');
        }
        $result = false;
        $payData = false;
        Db::startTrans();
        try {
            /*
             * 1.插入订单表
             * 2.插入订单详情表
             * 3.插入订单地址表
             * 4.下单减库存
             * 5.修改优惠券使用状态
             * 6.扣除用户使用的积分
             * */


            // 1.插入订单表
            $orderData = [
                'wid' => $this->wid,
                'user_id' => $this->auth->id,
                'order_no' => $order_no,
                'total_price' => $total_price, // 商品总金额(不含优惠折扣) 选择规格 * 购买数量
                'order_price' => $order_price,
                'coupon_id' => $coupon_id,
                'coupon_money' => $coupon_money,
                'score_money' => $score_money,
                'score_num' => $score,
                'score_bonus' => (int)$scoreGift,
                'pay_price' => $pay_price,
                'buyer_remark' => $buyer_remark,
                'pay_status' => 0,
                'express_price' => $deliveryMoney,
                'order_status' => 1,
                'platform' => $platform,
                'create_time' => date('Y-m-d H:i:s'),
                'update_time' => date('Y-m-d H:i:s')
            ];
            // 虚拟物品，默认已发货
            if ($goodsInfo->type == 0) {
                $orderData['delivery_status'] = 1;
                $orderData['delivery_time'] = date('Y-m-d H:i:s');
            }
            $order_id = (new OrderModel())->insertGetId($orderData);

            // 2.插入订单详情表
            $orderDetal = [
                'wid' => $this->wid,
                'order_id' => $order_id,
                'user_id' => $this->auth->id,
                'goods_id' => $goods_id,
                'goods_name' => $goodsInfo->getAttr('name'),
                'image' => $goodsSpec['image'] ? $goodsSpec['image'] : $goodsInfo->goods_images[0]['url'],
                'deduct_stock_type' => $goodsInfo->deduct_stock_type,
                'spec_type' => $goodsInfo->spec_type,
                'spec_id' => $goodsSpec['id'],
                'goods_sku_id' => $goodsSpec['spec_value_ids'],
                'goods_props' => $goodsSpec['spec_attr'],
                'content' => $goodsInfo->content,
                'goods_price' => $goodsSpec['goods_price'],
                'line_price' => $goodsSpec['line_price'],
                'goods_weight' => $goodsSpec['goods_weight'],
                'is_user_grade' => $is_user_grade,
                'grade_ratio' => $grade_ratio,
                'grade_goods_price' => $grade_goods_price,
                'grade_total_money' => $grade_total_money,
                'coupon_money' => $coupon_money,
                'score_money' => $score_money,
                'socre_num' => $score,
                'score_bonus' => (int)$scoreGift,
                'total_num' => $buy_num,
                'total_price' => $total_price,
                'total_pay_price' => $pay_price,
                'create_time' => date('Y-m-d H:i:s')
            ];
            $result = (new OrderDetailModel())->save($orderDetal);

            // 3.插入订单地址表
            $orderAddress = [
                'wid' => $this->wid,
                'order_id' => $order_id,
                'user_id' => $this->auth->id,
                'name' => $addressInfo['name'],
                'phone' => $addressInfo['phone'],
                'province_id' => $addressInfo['province_id'],
                'city_id' => $addressInfo['city_id'],
                'region_id' => $addressInfo['region_id'],
                'address' => $addressInfo['address'],
                'create_time' => date('Y-m-d H:i:s'),
            ];
            Db::table('cb_crm_order_address')->insert($orderAddress);

            // 4.下单减库存
            if ($goodsInfo->deduct_stock_type == 1) {
                // 下单减库存
                $goodsInfo->dec('stock_total', $buy_num)->update();
                Db::table('cb_crm_goods_spec')->where('goods_id', $goods_id)->where('id', $spec_id)->dec('stock_num', $buy_num)->update();
            }

            // 5.修改优惠券使用状态
            if ($coupon_id) {
                Db::table('cb_crm_user_coupon')->where('user_id', $this->auth->id)->where('coupon_id', $coupon_id)->update(['is_use' => 1]);
            }

            // 6.扣除用户使用的积分
            if ($score) {
                (new MemberModel())->where('id', $this->auth->id)->dec('score', $score)->update();
            }

            if ($pay_type == 'wechat') {
                // 订单微信支付
                $payData = $this->payWechat($orderData['pay_price'], $orderData['order_no'], $orderDetal['goods_name'], $platform);
                if (!$payData) {
                    throw new Exception('创建订单异常');
                }
            } elseif ($pay_type == 'balance') {
                // 余额支付
                $payResult = $this->payBalance($orderData['pay_price'], $orderData['order_no']);
                if (!$payResult) {
                    throw new Exception('余额支付异常');
                }
            }
            Db::commit();
        } catch (Exception $e) {
            Db::rollback();
            $this->error($e->getMessage());
        }
        if ($result) {
            $this->success('订单创建成功', $payData);
        } else {
            $this->error('订单提交失败');
        }
    }

    /*
     * 订单支付
     * */
    public function payOrder()
    {
        $order_id = $this->request->post('order_id');
        $platform = $this->request->post('platform', 'wechat');
        $pay_type = $this->request->post('pay_type', 'wechat');
        $order = (new OrderModel())->getDetail($order_id, $this->wid);

        if ($order->user_id != $this->auth->id || $order->pay_status != 0 && $order->order_status != 1) {
            $this->error('订单未找到');
        }
        $result = false;
        $payData = false;
        Db::startTrans();
        try {

            if ($pay_type == 'wechat') {
                $result = $this->payWechat($order->pay_price, $order->order_no, $order->goods[0]['goods_name'], $platform);
                if (!$result) {
                    throw new Exception('订单支付异常');
                }
            } elseif ($pay_type == 'balance') {
                $result = $this->payBalance($order->pay_price, $order->order_no);
                if (!$result) {
                    throw new Exception('余额支付异常');
                }
            }
            Db::commit();
        } catch (Exception $e) {
            Db::rollback();
            $this->error($e->getMessage());
        }
        if ($result) {
            $this->success('订单提交成功', $result);
        } else {
            $this->error('订单支付异常');
        }
    }

    /**
     * @title 微信支付
     * @param float $amount 订单金额
     * @param string $order_no 订单号
     * @param string $goods_name 商品名称
     * @param string $platform 支付平台 wechat 公众号   mpWechat 微信小程序
     * */
    protected function payWechat($amount, $order_no, $goods_name, $platform)
    {
        $config = (new SettingModel())->getConfigGroup('wechat', $this->wid);
        $payConfig = [
            // 必要配置
            'app_id' => $config['app_id'],
            'mch_id' => $config['mch_id'],
            'key' => $config['api_key'],   // API v2 密钥 (注意: 是v2密钥 是v2密钥 是v2密钥)
            'notify_url' => $this->request->domain() . '/home/api.notify/order/wid/' . $this->wid . '/platform/' . $platform,     // 你也可以在下单时单独设置来想覆盖它
        ];
        if ($platform == 'mpWechat') {
            $payConfig['app_id'] = $config['miniapp_id'];
            $payConfig['secret'] = $config['miniapp_secret'];
        }

        $openid = Db::table('cb_crm_user_third')->where('user_id', $this->auth->id)->where('platform', $platform)->value('openid');

        $isWechat = strpos($this->request->server('HTTP_USER_AGENT'), 'MicroMessenger') !== false;
        $isMiniProgram = strpos($this->request->server('HTTP_USER_AGENT'), 'MiniProgram') !== false;
        if ($isMiniProgram || $isWechat) {
            $trade_type = 'JSAPI';
        } else {
            $trade_type = 'MWEB';
        }

        $app = Factory::payment($payConfig);
        $payData = null;
        try {
            $result = $app->order->unify([
                'body' => '购买' . $goods_name,
                'openid' => $openid,
                'out_trade_no' => $order_no,
                'total_fee' => $amount * 100, // 单位：分
                'trade_type' => $trade_type, // 请对应换成你的支付方式对应的值类型
            ]);
            if (isset($result['return_code']) && $result['return_code'] === 'FAIL') {
                throw new Exception($result['return_msg']);
            }
            if (isset($result['result_code']) && $result['result_code'] === 'FAIL') {
                throw new Exception($result['err_code_des']);
            }
            if ($trade_type == 'JSAPI') {
                $jssdk = $app->jssdk;
                $payData = $jssdk->bridgeConfig($result['prepay_id']);
            };

        } catch (Exception $e) {
            $this->error($e->getMessage());
        } catch (\EasyWeChat\Kernel\Exceptions\InvalidArgumentException $e) {
            $this->error($e->getMessage());
        } catch (\EasyWeChat\Kernel\Exceptions\InvalidConfigException $e) {
            $this->error($e->getMessage());
        } catch (\GuzzleHttp\Exception\GuzzleException $e) {
            $this->error($e->getMessage());
        }

        return $payData;
    }


    /**
     * @title 余额支付
     * @param float $amount 订单金额
     * @param string $order_no 订单号
     * */
    protected function payBalance($amount, $order_no)
    {
        if ($this->auth->balance < $amount) {
            $this->error('可用余额不足');
        }
        $order = (new OrderModel())->where('order_no', $order_no)->find();
        if (!$order || $order->pay_status == 1) {
            $this->error('待支付订单不存在');
        }

        // 修改订单支付状态
        $order->pay_status = 1;
        $order->pay_type = 1;
        $order->pay_time = date('Y-m-d H:i:s');

        $res = $order->save();

        $orderDetailList = (new OrderDetailModel())->where('order_id', $order->id)->select();
        foreach ($orderDetailList as $key => $item) {
            // 付款减库存
            if ($item->deduct_stock_type == 2) {
                (new GoodsModel())->where('id', $item->goods_id)->dec('stock_total', $item->total_num)->update();
                Db::table('cb_crm_goods_spec')->where('goods_id', $item->goods_id)->where('id', $item->spec_id)->dec('stock_num', $item->total_num)->update();
            }
        }
        $userInfo = (new MemberModel())->where('id', $this->auth->id)->find();
        // 扣除用户余额
        $userInfo->balance = $userInfo->balance - $amount;

        $userInfo->save();

        if ($res) {

            // 发送短信通知
            $smsServices = app()->make(SmsServices::class);
            $mobile = (new SettingModel())->getAcceptPhone($this->wid);
            $smsServices->send($mobile,['type'=>'商品','order_no'=>$order_no],'后台新订单',$this->wid);

            return true;
        } else {
            return false;
        }
    }


    /*
     * 计算运费
     * */
    public function freight()
    {
        $goods_id = $this->request->get('goods_id');
        $spec_id = $this->request->get('spec_id');
        $buy_num = $this->request->get('buy_num', 1);
        $address_id = $this->request->get('address_id');

        // 获取商品信息
        $goodsInfo = (new GoodsModel())->getDetail($goods_id, $this->wid);
        if (!$goodsInfo || $goodsInfo->status != 1) {
            $this->error('购买的商品不存在');
        }

        // 运费
        $deliveryMoney = 0;

        // 虚拟物品  运费为 0
        if ($goodsInfo->type == 0) {
            $this->success('返回成功', compact('deliveryMoney'));
        }

        $addressInfo = Db::table('cb_crm_user_address')->where('id', $address_id)->find();

        $goodsSpec = Db::table('cb_crm_goods_spec')->where('goods_id', $goods_id)->where('id', $spec_id)->find();

        $total_price = $goodsSpec['goods_price'] * $buy_num;


        // 满额包邮
        $fullFreeConfig = (new SettingModel())->getConfigGroup('full_free', $this->wid);
        $isFreeDelivery = false; // 是否包邮
        if ($fullFreeConfig['is_open'] == 1 && $goodsInfo->type == 1) {
            $excludedGoodsIds = json_decode($fullFreeConfig['excludedGoodsIds'], true);
            $excludedRegions = json_decode($fullFreeConfig['excludedRegions'], true);

            if (!in_array($goods_id, $excludedGoodsIds) && !in_array($addressInfo['city'], $excludedRegions['cityIds']) && $total_price > $fullFreeConfig['money']) {
                $isFreeDelivery = true;
            }
        }
        if ($goodsInfo->type == 1 && $goodsInfo->delivery_id && !$isFreeDelivery) {
            $deliveryInfo = (new DeliveryModel())->with(['rule'])->where('id', $goodsInfo->delivery_id)->find();
            $deliveryRules = $deliveryInfo->rule;

            // 可配送区域
            $deliveryRegion = [];
            $ruleIndex = false;
            foreach ($deliveryRules as $key => $rule) {
                $region = json_decode($rule->region, true);
                $deliveryRegion = array_merge($deliveryRegion, $region);
                if (in_array($addressInfo['province_id'], $region) || in_array($addressInfo['city_id'], $region)) {
                    $ruleIndex = $key;
                }
            }
            if ($ruleIndex === false) {
                $this->error('当前收货地址不在配送范围');
            }
            // 计费方式:1=按件,2=按重
            if ($deliveryInfo->method == 1) {
                if ($buy_num <= $deliveryRules[$ruleIndex]['first_unit']) {
                    $deliveryMoney = $deliveryRules[$ruleIndex]['first_fee'];
                } else {
                    // 首件价格 +（（总数 - 首件数量） / 续件数量） * 续件价格
                    if ($deliveryRules[$ruleIndex]['additional'] > 0) {
                        $deliveryMoney = $deliveryRules[$ruleIndex]['first_fee'] + ($buy_num - $deliveryRules[$ruleIndex]['first_unit']) / $deliveryRules[$ruleIndex]['additional'] * $deliveryRules[$ruleIndex]['additional_fee'];
                    } else {
                        $deliveryMoney = $deliveryRules[$ruleIndex]['first_fee'];
                    }
                }
            } else {
                if (($goodsSpec->goods_weight * $buy_num) <= $deliveryRules[$ruleIndex]['first_unit']) {
                    $deliveryMoney = $deliveryRules[$ruleIndex]['first_fee'];
                } else {
                    // 首重价格 +（（总商品重量 - 首重） / 续重） * 续重价格
                    if ($deliveryRules[$ruleIndex]['additional'] > 0) {
                        $deliveryMoney = $deliveryRules[$ruleIndex]['first_fee'] + (($goodsSpec->goods_weight * $buy_num) - $deliveryRules[$ruleIndex]['first_unit']) / $deliveryRules[$ruleIndex]['additional'] * $deliveryRules[$ruleIndex]['additional_fee'];
                    } else {
                        $deliveryMoney = $deliveryRules[$ruleIndex]['first_fee'];
                    }
                }
            }
        }

        $this->success('返回成功', compact('deliveryMoney'));
    }
}
