<?php


namespace app\cbcrm\controller\api;


use app\cbcrm\model\config\SettingModel;
use app\cbcrm\model\member\RechargeModel;
use think\App;
use EasyWeChat\Factory;
use think\Exception;
use think\facade\Db;

class Recharge extends Base
{
    public function __construct(App $app)
    {
        parent::__construct($app);
        $this->auth = $this->auth();
    }

    public function recharge()
    {
        $config = (new SettingModel())->getConfigGroup('wechat', $this->wid);

        $amount = $this->request->post('amount');
        $platform = $this->request->post('platform');
        if ($amount < 0.01) {
            $this->error('充值金额必须大于1元');
        }

        $order_no = date('YmdHis') . mt_rand(100000, 999999);

        $openid = Db::table('cb_crm_user_third')->where('user_id', $this->auth->id)->where('platform', $platform)->value('openid');

        $isWechat = strpos($this->request->server('HTTP_USER_AGENT'), 'MicroMessenger') !== false;
        $isMiniProgram = strpos($this->request->server('HTTP_USER_AGENT'), 'MiniProgram') !== false;
        // 支付方式对应的值类型
        if ($isMiniProgram || $isWechat) {
            $trade_type = 'JSAPI';
        } else {
            $trade_type = 'MWEB';
        }

        if ($platform == 'wechat') {
            $payConfig = [
                //微信公众号 必要配置
                'app_id' => $config['app_id'],
                'mch_id' => $config['mch_id'],
                'key' => $config['api_key'],   // API v2 密钥 (注意: 是v2密钥 是v2密钥 是v2密钥)
                'notify_url' => $this->request->domain() . '/home/api.notify/recharge/wid/' . $this->wid . '/platform/' . $platform,     // 你也可以在下单时单独设置来想覆盖它
            ];
        } elseif ($platform == 'mpWechat') {
            $payConfig = [
                //微信小程序 必要配置
                'app_id' => $config['miniapp_id'],
                'secret' => $config['miniapp_secret'],
                'mch_id' => $config['mch_id'],
                'key' => $config['api_key'],   // API v2 密钥 (注意: 是v2密钥 是v2密钥 是v2密钥)
                'notify_url' => $this->request->domain() . '/home/api.notify/recharge/wid/' . $this->wid . '/platform/' . $platform,     // 你也可以在下单时单独设置来想覆盖它
            ];
        }


        $app = Factory::payment($payConfig);

        try {
            $result = $app->order->unify([
                'body' => '账户余额充值',
                'openid' => $openid,
                'out_trade_no' => $order_no,
                'total_fee' => $amount * 100, // 单位：分
                'trade_type' => $trade_type,
            ]);
            if (isset($result['return_code']) && $result['return_code'] === 'FAIL') {
                throw new Exception($result['return_msg']);
            }
            if (isset($result['result_code']) && $result['result_code'] === 'FAIL') {
                throw new Exception($result['err_code_des']);
            }
            if ($trade_type == 'JSAPI') {
                $jssdk = $app->jssdk;
                $result = $jssdk->bridgeConfig($result['prepay_id']);
            }

            // 写入充值表
            $data = [
                'wid' => $this->wid,
                'order_no' => $order_no,
                'user_id' => $this->auth->id,
                'type' => 1,
                'pay_money' => $amount,
                'gift_money' => 0,
                'actual_money' => $amount,
                'pay_status' => 0,
                'create_time' => date('Y-m-d H:i:s'),
                'update_time' => date('Y-m-d H:i:s'),
            ];
            (new RechargeModel())->save($data);

        } catch (Exception $e) {
            $this->error($e->getMessage());
        } catch (\EasyWeChat\Kernel\Exceptions\InvalidArgumentException $e) {
            $this->error($e->getMessage());
        } catch (\EasyWeChat\Kernel\Exceptions\InvalidConfigException $e) {
            $this->error($e->getMessage());
        } catch (\GuzzleHttp\Exception\GuzzleException $e) {
            $this->error($e->getMessage());
        };
        $this->success('返回成功', $result);
    }

    /*
     * 充值记录
     * */
    public function record()
    {
        $list = (new RechargeModel())->where('user_id', $this->auth->id)->where('pay_time', '>', 0)->order('id','desc')->select();

        $this->success('返回成功', $list);
    }
}
