<?php


namespace app\cbcrm\controller\api;


use app\cbcrm\model\config\SettingModel;
use app\cbcrm\model\member\MemberModel;
use app\cbcrm\model\trademark\ApplyerModel;
use app\cbcrm\model\trademark\OrderModel;
use app\cbcrm\model\trademark\ProductModel;
use app\cbcrm\services\SmsServices;
use EasyWeChat\Factory;
use think\App;
use think\Exception;
use think\facade\Db;

class TmRegister extends Base
{

    public function __construct(App $app)
    {
        parent::__construct($app);
        $this->auth = $this->auth();
    }


    /*
     * 订单提交
     * */
    public function order()
    {
        if ($this->request->isPost()) {
            $params = [];
            $params['product_id'] = $this->request->post('product_id') ?: $this->error('商标服务不存在');
            $params['applyer_id'] = $this->request->post('applyer_id', 0, 'intval') ?: $this->error('请选择商标申请人');
            $params['name'] = $this->request->post('name');
            $params['description'] = $this->request->post('description');
            $params['registration_image_file'] = $this->request->post('registration_image_file') ?: $this->error('请上传商标图样或生成图样');

            $this->request->filter('trim,strip_tags');
            $goodsData = $this->request->post('goodsData');

            $row = (new ProductModel())->where('id', $params['product_id'])->where('wid', $this->wid)->where('status', 1)->find();
            if (!$row) {
                $this->error('商标服务不存在');
            }
            $params['order_no'] = date('YmdHis') . rand(10000, 99999);
            $params['pay_status'] = 0;
            $goodsData = json_decode($goodsData, true);
            if (!$goodsData) {
                $this->error('请选择商标服务类别');
            }
            $params['user_id'] = $this->auth->id;
            $params['wid'] = $this->wid;
            $data = [];
            $orderMoney = 0;

            $resutl = false;
            Db::startTrans();
            try {
                foreach ($goodsData as $key => $item) {
                    $price = 0;
                    $childrenCount = count($item['children']);
                    $params['category'] = $item['id'];
                    if ($childrenCount <= 10) {
                        $price = $row->price;
                    } else {
                        $price = $row->price + (($childrenCount - 10) * $row->unit_price);
                    }
                    $params['status'] = '待审核';
                    $params['amount'] = $price;
                    $orderMoney += $price;
                    $children = [];
                    foreach ($item['children'] as $k => $child) {
                        foreach ($child as $value) {
                            $value['parent_id'] = $k;
                            array_push($children, $value);
                        }
                    }
                    $params['goods_data'] = json_encode($children, JSON_UNESCAPED_UNICODE);
                    $data[$key] = $params;
                }
                (new OrderModel())->saveAll($data);
                $resutl = [
                    'order_no' => $params['order_no'],
                    'order_money' => $orderMoney,
                    'balance' => $this->auth->balance
                ];
                Db::commit();
            } catch (Exception $e) {
                Db::rollback();
                $this->error($e->getMessage());
            }

            $this->success('订单创建成功', $resutl);
        }
    }




    public function payOrder()
    {
        $order_no = $this->request->post('order_no',0,'intval')?:$this->error('订单不存在');
        $platform = $this->request->post('platform', 'wechat');
        $pay_type = $this->request->post('pay_type', 'wechat');
        $orderList = (new OrderModel())->where('order_no',$order_no)->where('user_id',$this->auth->id)->select();

        $amount = 0.00;
        foreach ($orderList as $order) {
            if (!$order || $order->pay_status!=0) {
                $this->error('订单不存在');
            }else{
                $amount+=$order->amount;
            }
        }
        $result = false;
        Db::startTrans();
        try {

            if ($pay_type == 'wechat') {
                $result = $this->payWechat($amount, $order_no,'商标注册', $platform);
                if (!$result) {
                    throw new Exception('订单支付异常');
                }
            } elseif ($pay_type == 'balance') {
                $result = $this->payBalance($amount, $order_no);
                if (!$result) {
                    throw new Exception('余额支付异常');
                }
            }
            Db::commit();
        } catch (Exception $e) {
            Db::rollback();
            $this->error($e->getMessage());
        }
        if ($result) {
            $this->success('支付成功', $result);
        } else {
            $this->error('订单支付异常');
        }
    }
    /**
     * @title 微信支付
     * @param float $amount 订单金额
     * @param string $order_no 订单号
     * @param string $goods_name 商品名称
     * @param string $platform 支付平台 wechat 公众号   mpWechat 微信小程序
     * */
    protected function payWechat($amount, $order_no, $goods_name, $platform){
        $config = (new SettingModel())->getConfigGroup('wechat', $this->wid);
        $payConfig = [
            // 必要配置
            'app_id' => $config['app_id'],
            'mch_id' => $config['mch_id'],
            'key' => $config['api_key'],   // API v2 密钥 (注意: 是v2密钥 是v2密钥 是v2密钥)
            'notify_url' => $this->request->domain() . '/home/api.notify/trademark_order/wid/' . $this->wid . '/platform/' . $platform,     // 你也可以在下单时单独设置来想覆盖它
        ];
        if ($platform == 'mpWechat') {
            $payConfig['app_id'] = $config['miniapp_id'];
            $payConfig['secret'] = $config['miniapp_secret'];
        }

        $openid = Db::table('cb_crm_user_third')->where('user_id', $this->auth->id)->where('platform', $platform)->value('openid');

        $isWechat = strpos($this->request->server('HTTP_USER_AGENT'), 'MicroMessenger') !== false;
        $isMiniProgram = strpos($this->request->server('HTTP_USER_AGENT'), 'MiniProgram') !== false;
        if ($isMiniProgram || $isWechat) {
            $trade_type = 'JSAPI';
        } else {
            $trade_type = 'MWEB';
        }

        $app = Factory::payment($payConfig);
        $payData = null;
        try {
            $result = $app->order->unify([
                'body' => '购买' . $goods_name,
                'openid' => $openid,
                'out_trade_no' => $order_no,
                'total_fee' => $amount * 100, // 单位：分
                'trade_type' => $trade_type, // 请对应换成你的支付方式对应的值类型
            ]);
            if (isset($result['return_code']) && $result['return_code'] === 'FAIL') {
                throw new Exception($result['return_msg']);
            }
            if (isset($result['result_code']) && $result['result_code'] === 'FAIL') {
                throw new Exception($result['err_code_des']);
            }
            if ($trade_type == 'JSAPI') {
                $jssdk = $app->jssdk;
                $payData = $jssdk->bridgeConfig($result['prepay_id']);
            };

        } catch (Exception $e) {
            $this->error($e->getMessage());
        } catch (\EasyWeChat\Kernel\Exceptions\InvalidArgumentException $e) {
            $this->error($e->getMessage());
        } catch (\EasyWeChat\Kernel\Exceptions\InvalidConfigException $e) {
            $this->error($e->getMessage());
        } catch (\GuzzleHttp\Exception\GuzzleException $e) {
            $this->error($e->getMessage());
        }

        return $payData;
    }

    /**
     * @title 余额支付
     * @param float $amount 订单金额
     * @param string $order_no 订单号
     * */
    protected function payBalance($amount, $order_no)
    {
        if ($this->auth->balance < $amount) {
            $this->error('可用余额不足');
        }
        $orderList = (new OrderModel())->where('order_no', $order_no)->select();
        foreach ($orderList as $order) {
            if (!$order || $order->pay_status == 1) {
                $this->error('待支付订单不存在');
            }else{
                // 修改订单支付状态
                $order->pay_status = 1;
                $order->pay_time = date('Y-m-d H:i:s');
                $res = $order->save();
            }
        }


        $userInfo = (new MemberModel())->where('id', $this->auth->id)->find();
        // 扣除用户余额
        $userInfo->balance = $userInfo->balance - $amount;

        $userInfo->save();

        if ($res) {
            // 发送短信通知
            $smsServices = app()->make(SmsServices::class);
            $mobile = (new SettingModel())->getAcceptPhone($this->wid);
            $smsServices->send($mobile,['type'=>'商标注册','order_no'=>$order_no],'后台新订单',$this->wid);
            return true;
        } else {
            return false;
        }
    }


    /*
     * 获取商标申请人
     * */
    public function getApplyer()
    {
        $list = (new ApplyerModel())
            ->where('user_id', $this->auth->id)
            ->where('status', 1)
            ->field('id,name')
            ->order('id', 'desc')
            ->select();

        $this->success('返回成功', $list);
    }

    /*
     * 获取商标类别
     * */
    public function getClassesList()
    {
        $classesAliasList = (new OrderModel())->tmClassesAlias();
        $classesList = (new OrderModel())->tmClasses();

        $data = [];

        foreach ($classesAliasList as $key => $row) {
            $arr = [
                'id' => $key,
                'name' => $row,
                'itemsCount' => 0,
                'children' => []
            ];
            foreach ($classesList as $k => $item) {
                if ($k == $key) {
                    $arr['children'] = $item;
                    unset($classesList[$k]);
                }
            }
            $data[] = $arr;
        }

        foreach ($data as $key => $row) {
            foreach ($row['children'] as $ck => $child) {
                $data[$key]['children'][$ck]['itemsCount'] = 0;
                $data[$key]['children'][$ck]['open'] = false;
                foreach ($classesList as $k => $item) {
                    if ($k == $child['id']) {
                        $data[$key]['children'][$ck]['children'] = $item;
                        unset($classesList[$k]);
                    }
                }
            }
        }

        $this->success('返回成功', ['classesList' => $data]);
    }
}
