<?php


namespace app\cbcrm\controller\api;


use app\cbcrm\model\config\SettingModel;
use app\cbcrm\model\member\MemberModel;
use app\cbcrm\model\trademark\OrderModel;
use app\cbcrm\model\trademark\ProductModel;
use app\cbcrm\model\trademark\ServiceOrderModel;
use app\cbcrm\services\SmsServices;
use EasyWeChat\Factory;
use think\App;
use think\Exception;
use think\facade\Db;

class TmServiceOrder extends Base
{

    public function __construct(App $app)
    {
        parent::__construct($app);
        $this->auth = $this->auth();
    }

    /*
     * 用户中心商标服务订单列表
     * */
    public function userOrderList()
    {
        $status = $this->request->get('status', -1);
        $where = [
            ['wid', '=', $this->wid],
            ['user_id', '=', $this->auth->id],
        ];
        if ($status != -1) {
            $where[] = ['status','=', $status];
        }
        $list = (new ServiceOrderModel())
            ->where($where)
            ->order('create_time', 'desc')
            ->paginate(15);
        $this->success('返回成功', $list);
    }

    /*
     * 用户商标服务订单详情
     * */
    public function userOrderDetail()
    {
        $id = $this->request->get('id');
        $where = [
            ['wid', '=', $this->wid],
            ['user_id', '=', $this->auth->id],
            ['id', '=', $id],
        ];

        $row = (new ServiceOrderModel())->where($where)->find();

        if (!$row || $row->user_id != $this->auth->id) {
            $this->error('订单未找到');
        }

        $row->user = $this->auth;

        $this->success('返回成功',$row);
    }



    public function createOrder()
    {
        if ($this->request->isPost()) {
            $product_id = $this->request->post('product_id') ?: $this->error('参数错误');
            $params = [];
            $params['user_id'] = $this->auth->id;
            $params['wid'] = $this->wid;
            $params['order_no'] = date('YmdHis') . rand(10000, 99999);
            $params['pay_status'] = 0;
            $params['status'] = 0;
            $params['tm_id'] = $this->request->post('tm_id')?:$this->error('请填写商标申请号');
            $params['contact_name'] = $this->request->post('contact_name')?:$this->error('请填写联系人');
            $params['contact_tel'] = $this->request->post('contact_tel')?:$this->error('请填写联系电话');
            $params['message'] = $this->request->post('message');
            $row = (new ProductModel())->where('id', $product_id)->where('wid', $this->wid)->where('status', 1)->find();
            if (!$row) {
                $this->error('商标服务不存在');
            }
            $params['amount'] = $row->price;
            $params['product_info'] = json_encode($row,JSON_UNESCAPED_UNICODE);
            $params['create_time'] = date('Y-m-d H:i:s');
            $params['update_time'] = date('Y-m-d H:i:s');

            $order_id = (new ServiceOrderModel())->insertGetId($params);

            $data = [
                'order_id' => $order_id,
                'order_money' => $row->price,
                'balance' => $this->auth->balance
            ];

            $this->success('返回成功',$data);
        }
    }


    public function payOrder()
    {
        $order_id = $this->request->post('order_id',0,'intval')?:$this->error('订单不存在');
        $platform = $this->request->post('platform', 'wechat');
        $pay_type = $this->request->post('pay_type', 'wechat');
        $order = (new ServiceOrderModel())->where('id',$order_id)->where('user_id',$this->auth->id)->find();
        if (!$order || $order->status != 0) {
            $this->error('订单不存在',$order);
        }

        $productInfo = $order->product_info;

        $result = false;
        Db::startTrans();
        try {

            if ($pay_type == 'wechat') {
                $result = $this->payWechat($order->amount, $order->order_no,$productInfo['name'], $platform);
                if (!$result) {
                    throw new Exception('订单支付异常');
                }
            } elseif ($pay_type == 'balance') {
                $result = $this->payBalance($order->amount, $order->order_no);
                if (!$result) {
                    throw new Exception('余额支付异常');
                }
            }
            Db::commit();
        } catch (Exception $e) {
            Db::rollback();
            $this->error($e->getMessage());
        }
        if ($result) {
            $this->success('支付成功', $result);
        } else {
            $this->error('订单支付异常');
        }
    }
    /**
     * @title 微信支付
     * @param float $amount 订单金额
     * @param string $order_no 订单号
     * @param string $goods_name 商品名称
     * @param string $platform 支付平台 wechat 公众号   mpWechat 微信小程序
     * */
    protected function payWechat($amount, $order_no, $goods_name, $platform){
        $config = (new SettingModel())->getConfigGroup('wechat', $this->wid);
        $payConfig = [
            // 必要配置
            'app_id' => $config['app_id'],
            'mch_id' => $config['mch_id'],
            'key' => $config['api_key'],   // API v2 密钥 (注意: 是v2密钥 是v2密钥 是v2密钥)
            'notify_url' => $this->request->domain() . '/home/api.notify/tm_service_order/wid/' . $this->wid . '/platform/' . $platform,     // 你也可以在下单时单独设置来想覆盖它
        ];
        if ($platform == 'mpWechat') {
            $payConfig['app_id'] = $config['miniapp_id'];
            $payConfig['secret'] = $config['miniapp_secret'];
        }

        $openid = Db::table('cb_crm_user_third')->where('user_id', $this->auth->id)->where('platform', $platform)->value('openid');

        $isWechat = strpos($this->request->server('HTTP_USER_AGENT'), 'MicroMessenger') !== false;
        $isMiniProgram = strpos($this->request->server('HTTP_USER_AGENT'), 'MiniProgram') !== false;
        if ($isMiniProgram || $isWechat) {
            $trade_type = 'JSAPI';
        } else {
            $trade_type = 'MWEB';
        }

        $app = Factory::payment($payConfig);
        $payData = null;
        try {
            $result = $app->order->unify([
                'body' => '购买' . $goods_name,
                'openid' => $openid,
                'out_trade_no' => $order_no,
                'total_fee' => $amount * 100, // 单位：分
                'trade_type' => $trade_type, // 请对应换成你的支付方式对应的值类型
            ]);
            if (isset($result['return_code']) && $result['return_code'] === 'FAIL') {
                throw new Exception($result['return_msg']);
            }
            if (isset($result['result_code']) && $result['result_code'] === 'FAIL') {
                throw new Exception($result['err_code_des']);
            }
            if ($trade_type == 'JSAPI') {
                $jssdk = $app->jssdk;
                $payData = $jssdk->bridgeConfig($result['prepay_id']);
            };

        } catch (Exception $e) {
            $this->error($e->getMessage());
        } catch (\EasyWeChat\Kernel\Exceptions\InvalidArgumentException $e) {
            $this->error($e->getMessage());
        } catch (\EasyWeChat\Kernel\Exceptions\InvalidConfigException $e) {
            $this->error($e->getMessage());
        } catch (\GuzzleHttp\Exception\GuzzleException $e) {
            $this->error($e->getMessage());
        }

        return $payData;
    }

    /**
     * @title 余额支付
     * @param float $amount 订单金额
     * @param string $order_no 订单号
     * */
    protected function payBalance($amount, $order_no)
    {
        if ($this->auth->balance < $amount) {
            $this->error('可用余额不足');
        }
        $order = (new ServiceOrderModel())->where('order_no', $order_no)->find();
        if (!$order || $order->pay_status == 1) {
            $this->error('待支付订单不存在');
        }

        // 修改订单支付状态
        $order->status = 1;
        $order->pay_time = date('Y-m-d H:i:s');

        $res = $order->save();


        $userInfo = (new MemberModel())->where('id', $this->auth->id)->find();
        // 扣除用户余额
        $userInfo->balance = $userInfo->balance - $amount;

        $userInfo->save();

        if ($res) {
            // 发送短信通知
            $smsServices = app()->make(SmsServices::class);
            $mobile = (new SettingModel())->getAcceptPhone($this->wid);
            $smsServices->send($mobile,['type'=>'商标服务','order_no'=>$order_no],'后台新订单',$this->wid);
            return true;
        } else {
            return false;
        }
    }

}
