<?php


namespace app\cbcrm\controller\api;


use app\cbcrm\model\config\SettingModel;
use app\cbcrm\model\customer\CustomerModel;
use app\cbcrm\model\member\MemberModel;
use EasyWeChat\Kernel\Exceptions\InvalidConfigException;
use Ramsey\Uuid\Uuid;
use think\Exception;
use think\facade\Db;
use EasyWeChat\Factory;
use think\facade\Log;
use think\facade\Session;
use think\facade\Validate;

class User extends Base
{

    public function getUserInfo()
    {
        $token = $this->request->header('token');
        $user_id = Db::table('cb_crm_user_token')->where('token', $token)->where('expire_time','>',time())->value('user_id');
        if (!$user_id) {
            $this->error('请先登录','',401);
        }
        $userInfo = (new MemberModel())->with(['grade'])->where('id', $user_id)->find();
        $userInfo->coupon_count = Db::table('cb_crm_user_coupon')->where('user_id',$user_id)->count();
        unset($userInfo->password,$userInfo->salt);
        $this->success('返回成功', $userInfo);
    }

    // 账号密码登录
    public function login()
    {
        $account = $this->request->post('account')?:$this->error('请输入账号');
        $password = $this->request->post('password')?:$this->error('请输入密码');
        $platform = $this->request->post('platform');
        $openid = $this->request->post('openid');
        $userInfo = (new MemberModel())->where('username|mobile',$account)->find();
        if (!$userInfo) {
            $this->error('账号或密码不正确');
        }
        if (md5(md5($password) . $userInfo->salt) != $userInfo->password) {
            $this->error('账号或密码不正确');
        }
        unset($userInfo->password,$userInfo->salt);
        $token = Uuid::uuid1();
        $expire_time = time() + $this->expire_time;

        // 存储第三方登录平台openid
        if ($platform && $openid) {
            $userOpenid = Db::table('cb_crm_user_third')->where('user_id',$userInfo->id)->where('platform',$platform)->value('openid');
            if ($userOpenid && $openid!=$userOpenid) {
                Db::table('cb_crm_user_third')->where('user_id',$userInfo->id)->where('platform',$platform)->update(['openid'=>$openid]);
            }elseif(!$userOpenid){
                Db::table('cb_crm_user_third')->insert([
                    'user_id'=>$userInfo->id,
                    'platform'=>$platform,
                    'openid'=>$openid
                ]);
            }
        }

        Db::table('cb_crm_user_token')->insert(
            [
                'user_id'=>$userInfo->id,
                'token'=>"{$token}",
                'expire_time'=> $expire_time
            ]
        );
        $data = [
            'userInfo'=>$userInfo,
            'token'=>[
                'token'=>$token,
                'expire_time'=>$expire_time
            ]
        ];
        $this->success('登录成功',$data);
    }

    // 手机验证码登录
    public function mobilelogin()
    {
        $mobile = $this->request->post('mobile')?:$this->error('请输入手机号');
        $code = $this->request->post('code')?:$this->error('请输入验证码');
        $platform = $this->request->post('platform');
        $openid = $this->request->post('openid');

        $smsCode = Db::table('cb_crm_sms')
            ->where('event','login')
            ->where('mobile',$mobile)
            ->where('code',$code)
            ->find();
        if (!$smsCode || $smsCode['times']>0 || strtotime('-5 min') > strtotime($smsCode['create_time'])) {
            $this->error('验证码错误');
        }

        $userInfo = (new MemberModel())->where('mobile',$mobile)->find();
        if (!$userInfo) {
            // 用户不存在则进行注册
            $this->error('用户不存在');
        }

        // 删除验证码
        Db::table('cb_crm_sms')->where('id',$smsCode['id'])->delete();

        // 存储第三方登录平台openid
        if ($platform && $openid) {
            $userOpenid = Db::table('cb_crm_user_third')->where('user_id',$userInfo->id)->where('platform',$platform)->value('openid');
            if ($userOpenid && $openid!=$userOpenid) {
                Db::table('cb_crm_user_third')->where('user_id',$userInfo->id)->where('platform',$platform)->update(['openid'=>$openid]);
            }elseif(!$userOpenid){
                Db::table('cb_crm_user_third')->insert([
                    'user_id'=>$userInfo->id,
                    'platform'=>$platform,
                    'openid'=>$openid
                ]);
            }
        }

        unset($userInfo->password,$userInfo->salt);
        $token = Uuid::uuid1();
        $expire_time = time() + $this->expire_time;
        Db::table('cb_crm_user_token')->insert(
            [
                'user_id'=>$userInfo->id,
                'token'=>"{$token}",
                'expire_time'=> $expire_time
            ]
        );
        $data = [
            'userInfo'=>$userInfo,
            'token'=>[
                'token'=>$token,
                'expire_time'=>$expire_time
            ]
        ];
        $this->success('登录成功',$data);
    }

    public function register()
    {
        $username = $this->request->post('username')?:$this->error('请输入手机号');
        $code = $this->request->post('code')?:$this->error('请输入验证码');
        $password = $this->request->post('password')?:$this->error('请输入密码');
        $platform = $this->request->post('platform');
        $openid = $this->request->post('openid');
        if (!Validate::regex($username, "^1\d{10}$")) {
            $this->error('手机号格式错误');
        }
        $check = Db::table('cb_crm_user')->where('mobile', $username)->value('id');
        if ($check) {
            $this->error('该手机号已被注册');
        }

        $smsCode = Db::table('cb_crm_sms')
            ->where('event','register')
            ->where('mobile',$username)
            ->where('code',$code)
            ->find();
        if (!$smsCode || $smsCode['times']>0 || strtotime('-5 min') > strtotime($smsCode['create_time'])) {
            $this->error('验证码错误');
        }
        Db::startTrans();
        try {
            $salt = random_build('alnum',5);
            $data = [
                'wid'=>$this->wid,
                'username'=>$username,
                'avatar'=>import('static/img/avatar.png'),
                'password'=>md5(md5($password) . $salt),
                'salt'=>$salt,
                'mobile'=>$username,
                'nickname'=>substr_replace($username,'****',3,4),
                'grade_id'=>Db::table('cb_crm_user_grade')->where('wid',$this->wid)->order('weigh','asc')->value('id'),
                'create_time'=>date('Y-m-d H:i:s'),
                'update_time'=>date('Y-m-d H:i:s')
            ];
            $token = Uuid::uuid1();
            $expire_time = time() + $this->expire_time;

            $memberModel = new MemberModel();

            // 添加用户
            $user_id = $memberModel->insertGetId($data);

            // 存储第三方登录平台openid
            if ($platform && $openid) {
                Db::table('cb_crm_user_third')->insert([
                    'user_id'=>$user_id,
                    'platform'=>$platform,
                    'openid'=>$openid
                ]);
            }

            // 创建一个客户信息
            (new CustomerModel())->registerCustomer(['user_id'=>$user_id,'nickname'=>$data['nickname'],'mobile'=>$data['mobile']],$this->wid);

            $userInfo = $memberModel->where('id',$user_id)->find();
            // 删除验证码
            Db::table('cb_crm_sms')->where('id',$smsCode['id'])->delete();

            // 生成登录token
            Db::table('cb_crm_user_token')->insert(
                [
                    'user_id'=>$userInfo->id,
                    'token'=>"{$token}",
                    'expire_time'=> $expire_time
                ]
            );
            $data = [
                'userInfo'=>$userInfo,
                'token'=>[
                    'token'=>$token,
                    'expire_time'=>$expire_time
                ]
            ];
            Db::commit();
        } catch (Exception $e) {
            Db::rollback();
            $this->error($e->getMessage());
        }
        $this->success('注册成功',$data);
    }

    /*
     * 公众号登录
     * */
    public function mpLogin()
    {
        $page = $this->request->request('page');
        $wid = $this->request->request('wid');
        $red_url = Session::get('red_url');
        Session::set('wid',$wid);
        if (!$red_url) {
            $red_url = $this->request->domain() . '/home/web/wid/'.$wid.'/#/' . $page;
            Session::set('red_url', $red_url);
        }
        $config = (new SettingModel())->getConfigGroup('wechat',$wid);
        $config = [
            'app_id' => $config['app_id'],
            'secret' => $config['app_secret'],
            'oauth' => [
                'scopes'   => ['snsapi_userinfo'],
                'callback' => '/home/api.user/oauth_callback',
            ],
        ];

        $app = Factory::officialAccount($config);
        $redirectUrl = $app->oauth->scopes(['snsapi_userinfo'])
            ->redirect();
        return redirect($redirectUrl);
    }

    public function oauth_callback()
    {
        $red_url = Session::get('red_url');
        $wid = Session::get('wid');
        $config = (new SettingModel())->getConfigGroup('wechat',$wid);
        $config = [
            'app_id' => $config['app_id'],
            'secret' => $config['app_secret'],
            'oauth' => [
                'scopes'   => ['snsapi_userinfo'],
                'callback' => '/home/api.user/oauth_callback',
            ],
        ];
        $app = Factory::officialAccount($config);
        $oauth = $app->oauth;

        // 获取 OAuth 授权结果用户信息
        $code = $this->request->param('code');
        $user = $oauth->userFromCode($code);
        $openid =  $user->getId();
        $red_url .= '?openid=' . $openid;
        Session::set('red_url',null);
        Session::set('wid',null);
        Log::error($red_url);
        return redirect($red_url);
    }

    /*
     * 第三方登录
     * */
    public function third()
    {
        $platform = $this->request->request("platform");
        $code = $this->request->request("code");

        $config = (new SettingModel())->getConfigGroup('wechat',$this->wid);
        if ($platform == 'mpWechat') {
            $config = [
                'app_id' => $config['miniapp_id'],
                'secret' => $config['miniapp_secret']
            ];
        }

        $app = Factory::miniProgram($config);
        try {
            $result = $app->auth->session($code);
            if (isset($result['errcode']) && $result['errcode']!=0) {
                throw new Exception($result['errmsg']);
            }

            $this->success('返回成功',$result);

        } catch (InvalidConfigException $e) {
            $this->error($e->getMessage());
        } catch (Exception $e) {
            $this->error($e->getMessage());
        }
    }


}
