<?php
// +----------------------------------------------------------------------
// | [KtAdmin] Copyright (c) 2022 [ktadmin.cn] [kt8.cn]
// +----------------------------------------------------------------------
// | KtAdmin is NOT a free software, it under the license terms, visited http://www.kt8.cn/ for more details.
// +----------------------------------------------------------------------

declare (strict_types=1);

namespace app\cbcrm\controller\crm;

use app\cbcrm\controller\BaseUser;
use app\cbcrm\model\admin\site\MenuModel;
use app\cbcrm\model\admin\store\ConfigModel;
use app\cbcrm\model\article\ArticleModel;
use app\cbcrm\model\customer\CustomerModel;
use app\cbcrm\model\goods\CategoryModel;
use app\cbcrm\model\goods\GoodsModel;
use app\cbcrm\model\goods\OrderModel;
use think\Collection;
use think\facade\View;
use think\Request;

class Index extends BaseUser
{

    public function __construct(Request $request)
    {
        parent::__construct($request);
        $hasConfig = (new ConfigModel())->where('wid',$this->wid)->value('id');
        if (!$hasConfig) {
            (new ConfigModel())->insert([
                'wid'=>$this->wid
            ]);
        }
    }

    /*
     * 工作台
     * */
    public function dashboard()
    {
        // 客户数
        $customerTotal = (new CustomerModel())->where('wid', $this->wid)->count();
        // 订单数
        $orderTotal = (new OrderModel())->where('wid', $this->wid)
            ->where('order_status', '<>', 3)
            ->where('pay_status', 1)
            ->count();
        // 订单额
        $orderAmountTotal = (new OrderModel())->where('wid', $this->wid)
            ->where('order_status', '<>', 3)
            ->where('pay_status', 1)
            ->sum('pay_price');
        // 商品数
        $goodsTotal = (new GoodsModel())
            ->where('wid', $this->wid)
            ->count();
        // 文章数
        $articleTotal = (new ArticleModel())->where('wid', $this->wid)->count();
        // 下单客数
        $orderUserTotal = (new OrderModel())->where('wid', $this->wid)
            ->where('order_status', '<>', 3)
            ->where('pay_status', 1)
            ->group('user_id')
            ->count();
        // 待发货订单
        $deliverOrderTotal = (new OrderModel())->where('wid', $this->wid)
            ->where('order_status', '<>', 3)
            ->where('pay_status', 1)
            ->where('delivery_status', 0)
            ->count();
        // 待审核商标
        $auditTmOrderTotal = (new \app\cbcrm\model\trademark\OrderModel())->where('wid', $this->wid)
            ->where('pay_status', 1)
            ->where('status', '待审核')
            ->count();
        // 版权登记待审材料
        $auditCopyrightOrderTotal = (new \app\cbcrm\model\copyright\OrderModel())->where('wid', $this->wid)
            ->where('pay_status', 1)
            ->where('status', 1)
            ->count();
        // 待处理LOGO订单
        $logoOrderTotal = (new \app\cbcrm\model\logo\OrderModel())->where('wid', $this->wid)
            ->where('pay_status', 1)
            ->where('status', 0)
            ->count();
        // 最新客户
        $newCustomerList = (new CustomerModel())
            ->where('wid', $this->wid)
            ->field('id,name,create_time')
            ->limit(0, 5)
            ->order('create_time', 'desc')
            ->select();
        // 最新订单
        $newOrderList = (new OrderModel())
            ->with(['user', 'customer'])
            ->where('wid', $this->wid)
            ->where('order_status', '<>', 3)
            ->where('pay_status', 1)
            ->limit(0, 5)
            ->order('create_time', 'desc')
            ->select();
        // 近七天营业走势
        $startDate = date('Y-m-d 00:00:00', strtotime('-7 days'));
        $sevenDayOrder = (new OrderModel())
            ->where('wid', $this->wid)
            ->where('order_status', '<>', 3)
            ->where('pay_status', 1)
            ->whereBetweenTime('create_time', $startDate, date('Y-m-d H:i:s'))
            ->field('id,pay_price,create_time')
            ->select();
        $date = [];
        $starttime = strtotime($startDate);
        for ($start = 1; $start <= 7; $start++) {
            array_push($date, date('Y-m-d', strtotime("+{$start} days", $starttime)));
        }
        $echartData = [
            'orderNum' => [0, 0, 0, 0, 0, 0, 0],
            'orderAmount' => [0, 0, 0, 0, 0, 0, 0],
            'date' => $date
        ];
        foreach ($date as $key => $item) {
            foreach ($sevenDayOrder as $row) {
                $create_date = date('Y-m-d', strtotime($row->create_time));
                if ($item == $create_date) {
                    $echartData['orderNum'][$key]++;
                    $echartData['orderAmount'][$key] += $row->pay_price;
                }
            }
        }

        return success('返回成功', compact(
            'customerTotal',
            'orderTotal',
            'orderAmountTotal',
            'goodsTotal',
            'articleTotal',
            'orderUserTotal',
            'deliverOrderTotal',
            'newCustomerList',
            'newOrderList',
            'echartData', 'auditTmOrderTotal','auditCopyrightOrderTotal','logoOrderTotal'));

    }
}
