<?php


namespace app\cbcrm\controller\crm\config;


use app\cbcrm\controller\BaseUser;
use app\cbcrm\model\admin\sms\PackageModel;
use app\cbcrm\model\admin\sms\SignModel;
use app\cbcrm\model\admin\store\ConfigModel;
use app\cbcrm\model\config\SmsRechargeModel;
use app\cbcrm\services\PayService;
use think\Exception;
use think\facade\Db;
use think\Request;
use function Symfony\Component\Translation\t;

class PlatformSms extends BaseUser
{
    protected $model = null;

    public function __construct(Request $request)
    {
        parent::__construct($request);

        $this->model = new SmsRechargeModel();
    }

    /*
     * 初始数据
     * */
    public function init()
    {
        // 短信条数
        $store = (new ConfigModel())->where('wid', $this->wid)->field('sms_total,accept_phone')->find();
        $sms_total = $store['sms_total'];
        $accept_phone = $store['accept_phone'];
        // 充值套餐
        $packageList = (new PackageModel())->select();
        // 签名列表
        $signList = (new SignModel())->where('wid', $this->wid)->select();

        return success('返回成功', compact('sms_total', 'accept_phone', 'packageList', 'signList'));
    }

    /*
     * 充值记录
     * */
    public function getRechargeList()
    {
        $list = $this->model->where('wid', $this->wid)->where('status', 1)->paginate();
        return success('返回成功', $list);
    }

    /*
     * 购买短信套餐
     * */
    public function createOrder()
    {
        $package_id = $this->request->get('package_id', 0, 'intval');

        $row = (new PackageModel())->where('id', $package_id)->find();
        if (!$row) {
            return error('要购买的套餐已下架，请刷新后再试试');
        }
        Db::startTrans();
        $result = false;
        try {
            $data = [
                'wid' => $this->wid,
                'order_no' => date('YmdHis') . rand(10000, 99999),
                'type' => 1,
                'recharge_num' => $row->total_num,
                'price' => $row->price,
                'status' => 0
            ];
            $this->model->save($data);

            $notify_url = $this->request->domain() . '/home/api.notify/sms_recharge_order';

            $result = PayService::platformWechatPay($data['price'] * 100, $data['order_no'], '短信套餐充值', $notify_url);
            if ($result['status'] != 1) {
                throw new Exception($result['msg']);
            }
            Db::commit();
        } catch (Exception $e) {
            Db::rollback();
            return error($e->getMessage());
        }
        if (isset($result['data']) && $result['data']) {
            $url = $this->request->domain().'/home/api.index/qrcode?url=' . $result['data'];// 生成二维码链接
            return success('返回成功', ['url' => $url, 'order_no' => $data['order_no']]);
        } else {
            return error('订单创建失败');
        }

    }

    /*
     * 获取订单状态
     * */
    public function getOrderStatus()
    {
        $order_no = $this->request->get('order_no');

        $row = $this->model->where('wid', $this->wid)->where('order_no', $order_no)->field('id,status,pay_time')->find();

        //$row->pay_status = PayService::getPlatformOrderStatus($order_no);// 实时查询微信订单状态

        return success('返回成功', $row);
    }

    /*
     * 设置默认签名
     * */
    public function changeSign()
    {
        $sign_id = $this->request->post('sign_id', 0, 'intval');
        $accept_phone = $this->request->post('accept_phone');

        if ($sign_id) {
            (new SignModel())->where('wid', $this->wid)->update(['is_default' => 0]);
            (new SignModel())->where('wid', $this->wid)->where('id', $sign_id)->update(['is_default' => 1]);
        } else {
            (new SignModel())->where('wid', $this->wid)->update(['is_default' => 0]);
        }

        (new ConfigModel())->where('wid', $this->wid)->update(['accept_phone' => $accept_phone]);

        return success('修改成功');
    }

    public function addSign()
    {
        $params = \request()->post('form');
        $params['wid'] = $this->wid;
        $result = (new SignModel())->save($params);
        if ($result) {
            return success('返回成功', $result);
        } else {
            return error('返回失败');
        }
    }

    public function delSign()
    {
        $id = \request()->post('id', 0, 'intval');
        $row = (new SignModel())->where('wid', $this->wid)->where('id', $id)->find();

        if (!$row) {
            return error('要删除的签名不存在');
        }
        $result = $row->delete();

        if ($result) {
            return success('删除成功');
        } else {
            return error('删除失败');
        }
    }
}
