<?php


namespace app\cbcrm\controller\site;


use app\cbcrm\controller\BaseSite;
use app\cbcrm\model\copyright\OpusMaterialModel;
use app\cbcrm\model\copyright\OrderModel;
use app\cbcrm\model\copyright\ProductModel;
use app\cbcrm\model\copyright\SoftwareMaterialsModel;
use think\App;
use think\facade\Db;
use think\facade\View;

class CopyrightOrder extends BaseSite
{
    protected $model = null;

    public function __construct(App $app)
    {
        View::config(['layout_on'=>true,'layout_name' => 'site/common/user_layout']);
        parent::__construct($app);
        View::assign('title','版权登记');
    }

    public function index()
    {

        $copyright_id = $this->request->get('copyright_id', -1);
        $where = [
            ['user_id', '=', $this->auth()->id],
        ];
        if ($copyright_id != -1) {
            $where[] = ['copyright_id', '=', $copyright_id];
        }
        $list = (new OrderModel())
            ->with(['product'])
            ->where($where)
            ->order('create_time', 'desc')
            ->paginate(15);

        View::assign('list',$list);

        View::assign('page',$list->render());

        $productList = (new ProductModel())->where('wid',$this->wid)->where('status',1)->select();

        View::assign('productList',$productList);

        return View::fetch('cbcrm@site/user/copyright/order');
    }

    public function detail()
    {
        $id = $this->request->get('id',0,'intval');

        $user_id = $this->auth()->id;
        $where = [
            ['user_id', '=', $user_id],
            ['id', '=', $id],
        ];
        $this->model = new OrderModel();

        $row = $this->model->with(['product'])->where($where)->find();

        if (!$row || $row->user_id != $user_id) {
            $this->error('订单未找到');
        }

        $material = null;

        if ($row->product->form_type==0) {
            // 0=软著表单,1=作品著作权表单
            $materialModel = new SoftwareMaterialsModel();
            $material = $materialModel->where('order_id',$row->id)->find();
        }else{
            $materialModel = new OpusMaterialModel();
            $material = $materialModel->where('order_id',$row->id)->find();
        }

        $idCardType = $this->model->getIdCardType();
        $opusType = $this->model->getOpusType();
        $opusInditeType = $this->model->getOpusInditeType();
        $obtainType = $this->model->getObtainType();
        $rightOwnType = $this->model->getRightOwnType();
        $rightObtainMode = $this->model->getRightObtainMode();
        $inheritanceType = $this->model->getInheritanceType();
        $rightScopePart = $this->model->getRightScopePart();
        $softwareDescription = $this->model->getSoftwareDescription();
        $developMode = $this->model->getDevelopMode();
        $programLanguage = $this->model->getProgramLanguage();
        $specialtyTags = $this->model->getSpecialtyTags();

        View::assign(compact('rightOwnType','obtainType','opusInditeType','material','opusType','idCardType', 'rightObtainMode', 'inheritanceType', 'softwareDescription', 'developMode', 'rightScopePart', 'programLanguage', 'specialtyTags'));


        View::assign('row',$row);

        return View::fetch('cbcrm@site/user/copyright/order_detail');
    }

    public function del()
    {
        if ($this->request->isPost()) {
            $id = $this->request->post('id');
            $user_id = $this->auth()->id;
            $where = [
                ['user_id', '=', $user_id],
                ['id', '=', $id],
            ];

            $row = (new OrderModel())->where($where)->find();

            if (!$row || $row->user_id != $user_id) {
                $this->error('订单未找到');
            }

            $result = $row->delete();

            if ($result) {
                $this->success('删除成功');
            }else{
                $this->success('删除失败');
            }
        }
    }

    public function opus_material()
    {
        $id = $this->request->param('id');
        $user_id = $this->auth()->id;

        $this->model = new OrderModel();

        $where = [
            ['user_id', '=', $user_id],
            ['id', '=', $id],
        ];

        $row = $this->model->where($where)->find();

        if (!$row || $row->user_id != $user_id) {
            $this->error('订单未找到');
        }

        if ($row->pay_status != 1) {
            $this->error('当前订单未付款');
        }
        if ($row->status != 0) {
            $this->error('当前订单状态不允许该操作');
        }
        $materialModel = new OpusMaterialModel();

        $material = null;
        $material = $materialModel->where('order_id', $row->id)->where('user_id', $user_id)->find();

        $step = $this->request->param('step', 1, 'intval');
        $material_id = $this->request->param('material', 0, 'intval');
        if ($material_id) {
            $material = $materialModel->where(['id' => $material_id, 'user_id' => $user_id])->find();
            if (!$material) {
                $this->error('记录未找到');
            }
        }
        if ($material && $row->id != $material->order_id) {
            $this->error('数据异常', ['url'=>"/home/member/copyright/opus_material?id={$row->id}"]);
        }
        if ($step == 2 && (!$material || !$material->opusName)) {
            $this->error('参数异常',  ['url'=>"/home/member/copyright/opus_material?id={$row->id}"]);
        }

        if ($step == 3 && (!$material || !$material->owners)) {
            $this->error('参数异常',  ['url'=>"/home/member/copyright/opus_material?id={$row->id}"]);
        }

        if ($this->request->isPost()) {
            if ($step == 1) {
                $params = $this->request->post('row/a');
                if (($params['opusType'] == 'A' || $params['opusType'] == 'M') && !(int)$params['opusTypeDesc']) {
                    $this->error('请输入作品字数，且只能输入数字');
                }
                if ($params['opusType'] == 'L' && !$params['opusTypeDesc']) {
                    $this->error('请输入作品类别说明');
                }
                if ($params['opusType'] == 'I' && (!$params['opusTypeDesc'] || $params['opusTypeDesc'] == '0h0m0s' || $params['opusTypeDesc'] == 'h0m0s')) {
                    $this->error('请输入作品时长');
                }
                if ($params['publishStatus'] == 1 && (!$params['appearDate'] || !$params['appearCity'])) {
                    $this->error('已发表的作品请选择首次发表时间和发表地点');
                }
                $params['user_id'] = $user_id;
                $params['order_id'] = $row->id;
                $params['isDraft'] = 1;
                if ($material) {
                    // 存在则更新
                    $params['update_time'] = date('Y-m-d H:i:s');
                    $res = $materialModel->where('id', $material->id)->update($params);
                    $material_id = $material->id;
                } else {
                    // 新增
                    $res = $materialModel->save($params);
                    $material_id = $materialModel->id;
                }
                $this->success('提交成功',  ['url' => "/home/member/copyright/opus_material?id={$row->id}&step=2&material={$material_id}"]);

            } elseif ($step == 2) {
                $params = $this->request->post('row/a');
                $owners = $this->request->post('owners/a');
                $author = $this->request->post('author/a');

                if (!$material_id) {
                    $this->error('材料异常');
                }

                if ($params['obtainType'] == 'INHERIT' && (!$params['heirRelationshipProve'] || !$params['deathProve'] || !$params['validWill'])) {
                    $this->error('继承时,相关证明文件都必须上传');
                }
                if ($params['obtainType'] == 'REORGANIZATION' && (!$params['enterpriseChangeProve'] && !$params['contract'] && !$params['creditorsMeetingResolution'] && !$params['liquidationNotice'])) {
                    $this->error('承受时,相关证明文件至少上传一个');
                }
                if ($params['obtainType'] == 'TRANSFER' && (!$params['transferContract'] || !$params['rightObtainDate'])) {
                    $this->error('转让时,转让合同和权利取得日期必须上传/填写');
                }
                if ($params['rightOwnType'] == 'COT' && count($author) < 2) {
                    $this->error('合作作品作者至少填写2项，请继续添加作者');
                }
                if (count($params['rightScopePart']) == 0) {
                    $this->error('请选择权利拥有项');
                }
                $num = 1;
                $ownersData = [];
                foreach ($owners as $key => $owner) {
                    !$owner['cityName'] ? $this->error("请选择第{$num}个著作权人地区信息") : true;
                    !$owner['peopleType'] ? $this->error("请选择第{$num}个著作权人身份类别") : true;
                    !$owner['name'] ? $this->error("请填写第{$num}个著作权人姓名或名称") : true;
                    !$owner['idType'] ? $this->error("请选择第{$num}个著作权人证件类型") : true;
                    !$owner['idNumber'] ? $this->error("请填写第{$num}个著作权人证件号码") : true;

                    if (($owner['idType'] == '1' || $owner['idType'] == '21') && (!$owner['cardFront'] || !$owner['cardBack'])) {
                        $this->error("请完整上传第{$num}个著作权人证件材料附件");
                    } elseif ($owner['idType'] != '1' && $owner['idType'] != '21' && !$owner['cardFront']) {
                        $this->error("请上传第{$num}个著作权人证件材料附件");
                    }
                    !$owner['phone'] ? $this->error("请填写第{$num}个著作权人手机号码") : true;
                    !$owner['contactPerson'] ? $this->error("请填写第{$num}个著作权人联系人姓名") : true;
                    !$owner['email'] ? $this->error("请填写第{$num}个著作权人电子邮箱地址") : true;
                    $num++;
                    array_push($ownersData, $owner);
                }
                $params['owners'] = json_encode($ownersData, true);
                $num = 1;
                $authorData = [];
                foreach ($author as $item) {
                    !$item['name'] ? $this->error("请填写第{$num}个作者姓名/名称") : true;
                    array_push($authorData, $item);
                }
                $params['author'] = json_encode($authorData, true);
                $params['rightScopePart'] = implode( ',',$params['rightScopePart']);
                $params['update_time'] = date('Y-m-d H:i:s');
                $res = $materialModel->where('id', $material_id)->update($params);
                $this->success('提交成功', ['url' => "/home/member/copyright/opus_material?id={$row->id}&step=3&material={$material_id}"]);

            } elseif ($step == 3) {
                $material_id = $this->request->param('material', 0, 'intval');
                $material = $materialModel->where(['id' => $material_id, 'user_id' => $user_id])->find();
                if (!$material) {
                    $this->error('记录未找到');
                }
                $res = $materialModel->where('id', $material->id)->update(['isDraft' => 0]);
                $row->save(['status' => 1]);
                $this->success('材料提交成功',  ['msg' => '材料提交成功', 'url' => '/home/member/copyright/order.html']);

            }
        }

        $opusType = $this->model->getOpusType();
        $opusInditeType = $this->model->getOpusInditeType();
        $obtainType = $this->model->getObtainType();
        $rightOwnType = $this->model->getRightOwnType();
        $idCardType = $this->model->getIdCardType();
        $rightScopePart = $this->model->getOpusRightScopePart();

        View::assign(compact('row','material','step','opusType', 'opusInditeType', 'obtainType', 'rightOwnType', 'idCardType', 'rightScopePart'));


        return View::fetch('cbcrm@site/user/copyright/opus_material');
    }

    public function software_material()
    {

        $id = $this->request->param('id');
        $user_id = $this->auth()->id;

        $this->model = new OrderModel();

        $where = [
            ['user_id', '=', $user_id],
            ['id', '=', $id],
        ];

        $row = $this->model->where($where)->find();

        if (!$row || $row->user_id != $user_id) {
            $this->error('订单未找到');
        }

        if ($row->pay_status != 1) {
            $this->error('当前订单未付款');
        }
        if ($row->status != 0) {
            $this->error('当前订单状态不允许该操作');
        }

        $materialModel = new SoftwareMaterialsModel();
        $material = null;

        $material = $materialModel->where('order_id', $row->id)->where('user_id', $user_id)->find();

        $step = $this->request->param('step', 1, 'intval');
        $material_id = $this->request->param('material', 0, 'intval');
        if ($material_id) {
            $material = $materialModel->where(['id' => $material_id, 'user_id' => $user_id])->find();
            if (!$material) {
                $this->error('记录未找到');
            }
        }
        if ($material && $row->id != $material->order_id) {
            $this->error('数据异常', "/index/copyright/software_material?id={$row->id}");
        }
        if ($step == 2 && (!$material || !$material->softwareFullName)) {
            $this->error('参数异常', "/index/copyright/software_material?id={$row->id}");
        }

        if ($step == 3 && (!$material || !$material->owners)) {
            $this->error('参数异常', "/index/copyright/software_material?id={$row->id}");
        }
        if ($step == 4 && (!$material || !$material->hardwareDevelopEnv)) {
            $this->error('参数异常', "/index/copyright/software_material?id={$row->id}");
        }
        if ($this->request->isPost()) {
            if ($step==1) {
                $params = $this->request->post('row/a');
                if (!$params['softwareFullName']) {
                    $this->error('请输入软件全称');
                }
                if (!$params['version']) {
                    $this->error('请输入版本号');
                }
                if ($params['isReg'] == '1' && !$params['inheritanceOrigRegNumber']) {
                    $this->error('请输入正确的原登记号');
                }
                if ($params['isReg'] == '1' && !$params['inheritanceOrigRegPaper']) {
                    $this->error('请上传原登记证书扫描件');
                }
                if ($params['isChange'] == '1' && !$params['inheritanceOrigChangeNumber']) {
                    $this->error('请输入变更或补充证明编号');
                }
                if ($params['rightScope'] == 2 && !implode(',', $params['rightScopePart'])) {
                    $this->error('请选择权利范围');
                }
                if ($params['rightScope'] == 2 && !$params['rightScopePartPaper']) {
                    $this->error('请上传权利范围相关文件');
                }
                if(isset($params['rightScopePart'])){
                    $params['rightScopePart'] = implode(',', $params['rightScopePart']);
                }


                $params['user_id'] = $user_id;
                $params['order_id'] = $row->id;
                $params['isDraft'] = 1;
                if ($material) {
                    // 存在则更新
                    $params['update_time'] = date('Y-m-d H:i:s');
                    $res = $materialModel->where('id', $material->id)->update($params);
                    $material_id = $material->id;
                } else {
                    // 新增
                    $res = $materialModel->save($params);
                    $material_id = $materialModel->id;
                }
                $this->success('提交成功',  ['url' => "/home/member/copyright/software_material?id={$row->id}&step=2&material={$material_id}"]);
            } elseif ($step==2) {
                $params = $this->request->post('row/a');
                $owners = $this->request->post('owners/a');
                if ($params['softwareDescription']=='MODIFY' && !$params['modifyDescription']) {
                    $this->error('请填写修改合成或翻译说明');
                }
                if ((isset($params['isReg']) && $params['isReg']==1) && (!$params['origRegisterNumber'] || !$params['origRegisterPaper'])) {
                    $this->error('请填写原有软件登记号和上传原登记证书扫描件');
                }
                if (isset($params['isModifyAuth']) && $params['isModifyAuth']==1 && !$params['modifyAuthPaper']) {
                    $this->error('请上传授权书');
                }
                if ($params['developMode']!='SINGLE' && !$params['cooperateContract']) {
                    $this->error('请上传合作开发/委托/项目合同');
                }
                if ($params['publishStatus']==1 && (!$params['appearDate'] || !$params['appearCity'])) {
                    $this->error('请填写发表日期和首次发表地点');
                }
                $num = 1;
                $ownersData = [];
                foreach ($owners as $key => $owner) {
                    !$owner['cityName'] ? $this->error("请填写第{$num}个著作权人地区信息") : true;
                    !$owner['peopleType'] ? $this->error("请选择第{$num}个著作权人身份类别") : true;
                    !$owner['name'] ? $this->error("请填写第{$num}个著作权人姓名或名称") : true;
                    !$owner['idType'] ? $this->error("请选择第{$num}个著作权人证件类型") : true;
                    !$owner['idNumber'] ? $this->error("请填写第{$num}个著作权人证件号码") : true;
                    $num++;
                    array_push($ownersData, $owner);
                }
                $params['owners'] = json_encode($ownersData, true);

                $params['update_time'] = date('Y-m-d H:i:s');
                $res = $materialModel->where('id', $material_id)->update($params);
                $this->success('提交成功',  ['url' => "/home/member/copyright/software_material?id={$row->id}&step=3&material={$material_id}"]);
            }elseif ($step==3){
                $params = $this->request->post('row/a');
                $programIdentifyMaterial = $this->request->post('programIdentifyMaterial/a');
                $documentIdentifyMaterial = $this->request->post('documentIdentifyMaterial/a');
                $params['update_time'] = date('Y-m-d H:i:s');
                $params['programIdentifyMaterial'] = json_encode($programIdentifyMaterial,true);
                $documentIdentifyMaterialData = [];
                foreach ($documentIdentifyMaterial as $item) {
                    array_push($documentIdentifyMaterialData,$item);
                }
                $params['programLanguage'] = implode(',',$params['programLanguage']);
                $params['specialtyTags'] = implode(',',$params['specialtyTags']);
                $params['documentIdentifyMaterial'] = json_encode($documentIdentifyMaterialData,true);
                $params['update_time'] = date('Y-m-d H:i:s');
                $res = $materialModel->where('id', $material_id)->update($params);
                $this->success('提交成功',  ['url' => "/home/member/copyright/software_material?id={$row->id}&step=4&material={$material_id}"]);
            } elseif ($step==4) {
                $params = $this->request->post('row/a');
                $materialList = $this->request->post('materialList/a');
                $params['update_time'] = date('Y-m-d H:i:s');
                if (!$params['isDraft']==1) {
                    // 非草稿箱验证
                    foreach ($materialList as $item) {
                        foreach ($item as $value) {
                            if (intval($value['materialPages'])==0) {
                                $this->error("请填写{$value['materialName']}材料页数");
                            }
                        }
                    }
                    $row->save(['status'=>1]);
                }
                $params['materialList'] = json_encode($materialList,true);
                $res = $materialModel->where('id', $material->id)->update($params);
                if ($res) {
                    $this->success('材料提交成功', ['msg' => '材料提交成功', 'url' => '/home/member/copyright/order.html']);
                } else {
                    $this->error('材料提交失败');
                }
            }
        }

        $idCardType = $this->model->getIdCardType();
        $rightObtainMode = $this->model->getRightObtainMode();
        $inheritanceType = $this->model->getInheritanceType();
        $rightScopePart = $this->model->getRightScopePart();
        $softwareDescription = $this->model->getSoftwareDescription();
        $developMode = $this->model->getDevelopMode();
        $programLanguage = $this->model->getProgramLanguage();
        $specialtyTags = $this->model->getSpecialtyTags();
        View::assign('row', $row);
        View::assign(compact('material','step','idCardType', 'rightObtainMode', 'inheritanceType', 'softwareDescription', 'developMode', 'rightScopePart', 'programLanguage', 'specialtyTags'));

        return View::fetch('cbcrm@site/user/copyright/software_material');
    }
}
