<?php

use app\cbcrm\middleware\check;
use app\cbcrm\middleware\membercheck;
use think\facade\Route;



/*
 * CRM 商户端接口
 * */
Route::group('crm', function () {

    Route::group('user', function () {
        Route::any('/index', 'crm.user.index/index'); //首页
        Route::any('/getUserInfo', 'crm.user.index/getUserInfo');// 获取用户信息
    })->middleware(app\cbcrm\middleware\check::class);

    Route::post('user/login', 'crm.user.index/login');// 登录
    Route::post('user/logout', 'crm.user.index/logout');// 注销登录
    Route::get('basic/info', 'crm.basic/getAppInfo')->middleware(app\cbcrm\middleware\check::class);// 获取应用信息
    Route::get('basic/getRegionTree', 'crm.basic/getRegionTree');// 获取地区树

    Route::group('config', function () {
        // 数据字典
        Route::get('/dictionary/index', 'crm.config.dictionary/index');
        Route::get('/dictionary/getType', 'crm.config.dictionary/getType');
        Route::post('/dictionary/add', 'crm.config.dictionary/add');
        Route::post('/dictionary/edit', 'crm.config.dictionary/edit');
        Route::delete('/dictionary/del', 'crm.config.dictionary/del');
        // 运费模板
        Route::get('/delivery/index', 'crm.config.delivery/index');
        Route::get('/delivery/detail', 'crm.config.delivery/detail');
        Route::get('/delivery/all', 'crm.config.delivery/all');
        Route::post('/delivery/add', 'crm.config.delivery/add');
        Route::post('/delivery/edit', 'crm.config.delivery/edit');
        Route::delete('/delivery/del', 'crm.config.delivery/del');
        // 系统设置
        Route::get('/setting/config', 'crm.config.setting/getConfig');
        Route::post('/setting/update', 'crm.config.setting/update');
        // 物流公司
        Route::get('/express/index', 'crm.config.express/index');
        Route::get('/express/all', 'crm.config.express/getAll');
        Route::post('/express/add', 'crm.config.express/add');
        Route::post('/express/edit', 'crm.config.express/edit');
        Route::delete('/express/del', 'crm.config.express/del');
        // 地址管理
        Route::get('/address/index', 'crm.config.address/index');
        Route::get('/address/all', 'crm.config.address/getAll');
        Route::post('/address/add', 'crm.config.address/add');
        Route::post('/address/edit', 'crm.config.address/edit');
        Route::delete('/address/del', 'crm.config.address/del');
        // 导航设置
        Route::get('/nav/index', 'crm.config.nav/index');
        Route::post('/nav/add', 'crm.config.nav/add');
        Route::post('/nav/edit', 'crm.config.nav/edit');
        Route::delete('/nav/del', 'crm.config.nav/del');

        Route::get('smstemplate/index', 'crm.config.SmsTemplate/index');
        Route::post('smstemplate/edit', 'crm.config.SmsTemplate/edit');

    })->middleware(app\cbcrm\middleware\check::class);

    Route::group('customer', function () {
        // 客户
        Route::get('/customer/index', 'crm.customer.customer/index');
        Route::get('/customer/getFormData', 'crm.customer.customer/getFormData');
        Route::get('/customer/checkCustomer', 'crm.customer.customer/checkCustomer');
        Route::post('/customer/add', 'crm.customer.customer/add');
        Route::get('/customer/detail', 'crm.customer.customer/detail');
        Route::post('/customer/edit', 'crm.customer.customer/edit');
        Route::post('/customer/assgin', 'crm.customer.customer/assgin');
        Route::delete('/customer/del', 'crm.customer.customer/del');
        // 联系人
        Route::get('/contacts/index', 'crm.customer.contacts/index');
        Route::get('/contacts/getFormData', 'crm.customer.contacts/getFormData');
        Route::post('/contacts/add', 'crm.customer.contacts/add');
        Route::post('/contacts/edit', 'crm.customer.contacts/edit');
        Route::delete('/contacts/del', 'crm.customer.contacts/del');
        Route::get('/contacts/customer', 'crm.customer.contacts/getCustomerContact');
        // 跟单记录
        Route::get('/follow/index', 'crm.customer.follow/index');
        Route::get('/follow/getFormData', 'crm.customer.follow/getFormData');
        Route::post('/follow/add', 'crm.customer.follow/add');
        Route::post('/follow/edit', 'crm.customer.follow/edit');
        Route::delete('/follow/del', 'crm.customer.follow/del');
        // 客户商机
        Route::get('/niche/index', 'crm.customer.niche/index');
        Route::get('/niche/getFormData', 'crm.customer.niche/getFormData');
        Route::post('/niche/add', 'crm.customer.niche/add');
        Route::post('/niche/edit', 'crm.customer.niche/edit');
        Route::delete('/niche/del', 'crm.customer.niche/del');

        // 客户材料
        Route::get('/attachment/index', 'crm.customer.attachment/index');
        Route::post('/attachment/add', 'crm.customer.attachment/add');
        Route::post('/attachment/edit', 'crm.customer.attachment/edit');
        Route::delete('/attachment/del', 'crm.customer.attachment/del');

        // 操作记录
        Route::get('/operate/index', 'crm.customer.operate/index');

    })->middleware(app\cbcrm\middleware\check::class);

    Route::group('goods', function () {
        // 商品分类
        Route::get('/category/index', 'crm.goods.category/index');
        Route::post('/category/add', 'crm.goods.category/add');
        Route::post('/category/edit', 'crm.goods.category/edit');
        Route::delete('/category/del', 'crm.goods.category/del');
        // 商品列表
        Route::get('/goods/index', 'crm.goods.goods/index');
        Route::get('/goods/listByIds', 'crm.goods.goods/listByIds');
        Route::get('/goods/detail', 'crm.goods.goods/detail');
        Route::post('/goods/add', 'crm.goods.goods/add');
        Route::post('/goods/edit', 'crm.goods.goods/edit');
        Route::delete('/goods/del', 'crm.goods.goods/del');
        // 订单列表
        Route::get('/order/index', 'crm.goods.order/index');
        Route::get('/order/detail', 'crm.goods.order/detail');
        Route::delete('/order/del', 'crm.goods.order/del');
        Route::post('/order/delivery', 'crm.goods.order/delivery');
        Route::post('/order/confirmCancel', 'crm.goods.order/confirmCancel');
        Route::get('/order/export', 'crm.goods.order/export');

        // 售后单列表
        Route::get('/order_refund/index', 'crm.goods.order_refund/index');
        Route::get('/order_refund/detail', 'crm.goods.order_refund/detail');
        Route::post('/order_refund/audit', 'crm.goods.order_refund/audit');
        Route::post('/order_refund/receipt', 'crm.goods.order_refund/receipt');
        Route::delete('/order_refund/del', 'crm.goods.order_refund/del');

    })->middleware(app\cbcrm\middleware\check::class);

    Route::group('attachment', function () {
        // 文件分组
        Route::get('/group_add', 'crm.attachment/group_add');
        Route::post('/move_group', 'crm.attachment/move_group');
        Route::get('/group_list', 'crm.attachment/group_list');
        // 文件列表
        Route::get('/list', 'crm.attachment/index');
        Route::get('/upload_image', 'crm.attachment/upload_image');
        Route::post('/upload_image', 'crm.attachment/upload_image');
    })->middleware(app\cbcrm\middleware\check::class);

    Route::group('member', function () {
        // 会员列表
        Route::get('/member/index', 'crm.member.member/index');
        Route::post('/member/grade', 'crm.member.member/grade');
        Route::post('/member/recharge', 'crm.member.member/recharge');
        // 余额明细
        Route::get('/balance/log', 'crm.member.member/balanceLog');
        // 积分明细
        Route::get('/score/log', 'crm.member.member/scoreLog');
        // 充值记录
        Route::get('recharge/index', 'crm.member.recharge/index');

        // 会员等级
        Route::get('/grade/index', 'crm.member.grade/index');
        Route::get('/grade/all', 'crm.member.grade/all');
        Route::post('/grade/add', 'crm.member.grade/add');
        Route::post('/grade/edit', 'crm.member.grade/edit');
        Route::delete('/grade/del', 'crm.member.grade/del');

    })->middleware(app\cbcrm\middleware\check::class);


    Route::group('article', function () {
        // 文章分类
        Route::get('/category/index', 'crm.article.category/index');
        Route::post('/category/add', 'crm.article.category/add');
        Route::post('/category/edit', 'crm.article.category/edit');
        Route::delete('/category/del', 'crm.article.category/del');


        // 文章列表
        Route::get('/article/index', 'crm.article.article/index');
        Route::get('/article/detail', 'crm.article.article/detail');
        Route::post('/article/add', 'crm.article.article/add');
        Route::post('/article/edit', 'crm.article.article/edit');
        Route::delete('/article/del', 'crm.article.article/del');

        // 广告列表
        Route::get('/block/index', 'crm.article.block/index');
        Route::post('/block/add', 'crm.article.block/add');
        Route::post('/block/edit', 'crm.article.block/edit');
        Route::delete('/block/del', 'crm.article.block/del');
        Route::get('/block/detail', 'crm.article.block/detail');

        // 通知公告
        Route::get('/notice/index', 'crm.article.notice/index');
        Route::post('/notice/add', 'crm.article.notice/add');
        Route::post('/notice/edit', 'crm.article.notice/edit');
        Route::delete('/notice/del', 'crm.article.notice/del');

        // 友情链接
        Route::get('/friendlink/index', 'crm.article.friendLink/index');
        Route::post('/friendlink/add', 'crm.article.friendLink/add');
        Route::post('/friendlink/edit', 'crm.article.friendLink/edit');
        Route::delete('/friendlink/del', 'crm.article.friendLink/del');

        // 菜单列表
        Route::get('/menu/index', 'crm.article.menu/index');
        Route::post('/menu/add', 'crm.article.menu/add');
        Route::post('/menu/edit', 'crm.article.menu/edit');
        Route::post('/menu/sync', 'crm.article.menu/sync');
        Route::delete('/menu/del', 'crm.article.menu/del');

    })->middleware(app\cbcrm\middleware\check::class);

    Route::group('market', function () {
        // 充值套餐
        Route::get('/recharge_package/index', 'crm.market.recharge_package/index');
        Route::post('/recharge_package/add', 'crm.market.recharge_package/add');
        Route::post('/recharge_package/edit', 'crm.market.recharge_package/edit');
        Route::delete('/recharge_package/del', 'crm.market.recharge_package/del');

        Route::get('/coupon/index', 'crm.market.coupon/index');
        Route::post('/coupon/add', 'crm.market.coupon/add');
        Route::post('/coupon/edit', 'crm.market.coupon/edit');
        Route::delete('/coupon/del', 'crm.market.coupon/del');
        Route::get('/coupon/detail', 'crm.market.coupon/detail');
        Route::get('/coupon/receive', 'crm.market.coupon/receive');

    })->middleware(app\cbcrm\middleware\check::class);


    Route::group('report', function () {
        // 经营概况
        Route::get('/management', 'crm.report.management/index');

    })->middleware(app\cbcrm\middleware\check::class);

    Route::group('trademark', function () {
        // 商标产品
        Route::get('/product/index', 'crm.trademark.product/index');
        Route::get('/product/detail', 'crm.trademark.product/detail');
        Route::post('/product/add', 'crm.trademark.product/add');
        Route::post('/product/edit', 'crm.trademark.product/edit');
        Route::delete('/product/del', 'crm.trademark.product/del');
        Route::post('/product/sync', 'crm.trademark.product/sync');
        // 商标申请人
        Route::get('/applyer/index', 'crm.trademark.applyer/index');
        Route::get('/applyer/detail', 'crm.trademark.applyer/detail');
        Route::post('/applyer/edit', 'crm.trademark.applyer/edit');
        Route::delete('/applyer/del', 'crm.trademark.applyer/del');
        // 商标订单
        Route::get('/order/index', 'crm.trademark.order/index');
        Route::get('/order/detail', 'crm.trademark.order/detail');
        Route::post('/order/edit', 'crm.trademark.order/edit');
        Route::delete('/order/del', 'crm.trademark.order/del');
        // 商标发文
        Route::get('/file/index', 'crm.trademark.file/index');
        Route::post('/file/add', 'crm.trademark.file/add');
        Route::get('/file/detail', 'crm.trademark.file/detail');
        Route::post('/file/edit', 'crm.trademark.file/edit');
        Route::delete('/file/del', 'crm.trademark.file/del');
        // 发文索取
        Route::get('/fileFetch/index', 'crm.trademark.fileFetch/index');
        Route::post('/fileFetch/add', 'crm.trademark.fileFetch/add');
        Route::get('/fileFetch/detail', 'crm.trademark.fileFetch/detail');
        Route::post('/fileFetch/edit', 'crm.trademark.fileFetch/edit');
        Route::delete('/fileFetch/del', 'crm.trademark.fileFetch/del');


        // 商标服务订单
        Route::get('/service_order/index', 'crm.trademark.service_order/index');
        Route::get('/service_order/detail', 'crm.trademark.service_order/detail');
        Route::post('/service_order/edit', 'crm.trademark.service_order/edit');
        Route::delete('/service_order/del', 'crm.trademark.service_order/del');

    })->middleware(app\cbcrm\middleware\check::class);

    Route::group('copyright', function () {
        // 版权登记
        Route::get('/product/index', 'crm.copyright.product/index');
        Route::get('/product/detail', 'crm.copyright.product/detail');
        Route::post('/product/add', 'crm.copyright.product/add');
        Route::post('/product/edit', 'crm.copyright.product/edit');
        Route::delete('/product/del', 'crm.copyright.product/del');
        // 版权订单
        Route::get('/order/index', 'crm.copyright.order/index');
        Route::get('/order/softwareDetail', 'crm.copyright.order/softwareDetail');
        Route::get('/order/opusDetail', 'crm.copyright.order/opusDetail');
        Route::get('/order/detail', 'crm.copyright.order/detail');
        Route::post('/order/add', 'crm.copyright.order/add');
        Route::post('/order/edit', 'crm.copyright.order/edit');
        Route::delete('/order/del', 'crm.copyright.order/del');

    })->middleware(app\cbcrm\middleware\check::class);

    Route::group('logo', function () {
        // logo 案例
        Route::get('/case/index', 'crm.logo.logo_case/index');
        Route::get('/case/detail', 'crm.logo.logo_case/detail');
        Route::get('/case/formData', 'crm.logo.logo_case/getFormData');
        Route::post('/case/add', 'crm.logo.logo_case/add');
        Route::post('/case/edit', 'crm.logo.logo_case/edit');
        Route::delete('/case/del', 'crm.logo.logo_case/del');
        // logo 配套VI
        Route::get('/logovi/index', 'crm.logo.logo_vi/index');
        Route::get('/logovi/detail', 'crm.logo.logo_vi/detail');
        Route::post('/logovi/add', 'crm.logo.logo_vi/add');
        Route::post('/logovi/edit', 'crm.logo.logo_vi/edit');
        Route::delete('/logovi/del', 'crm.logo.logo_vi/del');
        // logo 套餐
        Route::get('/package/index', 'crm.logo.package/index');
        Route::get('/package/detail', 'crm.logo.package/detail');
        Route::post('/package/add', 'crm.logo.package/add');
        Route::post('/package/edit', 'crm.logo.package/edit');
        Route::delete('/package/del', 'crm.logo.package/del');
        // logo 订单
        Route::get('/order/index', 'crm.logo.order/index');
        Route::get('/order/detail', 'crm.logo.order/detail');
        Route::post('/order/add', 'crm.logo.order/add');
        Route::post('/order/edit', 'crm.logo.order/edit');
        Route::delete('/order/del', 'crm.logo.order/del');
        // logo 附加服务
        Route::get('/other/index', 'crm.logo.other/index');
        Route::post('/other/add', 'crm.logo.other/add');
        Route::post('/other/edit', 'crm.logo.other/edit');
        Route::delete('/other/del', 'crm.logo.other/del');

    })->middleware(app\cbcrm\middleware\check::class);

    Route::group('manage', function () {
        // 部门管理
        Route::get('/department/index', 'crm.manage.department/index');
        Route::get('/department/list', 'crm.manage.department/getDepartmentList');
        Route::get('/department/staff', 'crm.manage.department/getDepartmentStaffList');
        Route::post('/department/add', 'crm.manage.department/add');
        Route::post('/department/edit', 'crm.manage.department/edit');
        Route::delete('/department/del', 'crm.manage.department/del');
        // 职位角色管理
        Route::get('/role/index', 'crm.manage.role/index');
        Route::post('/role/add', 'crm.manage.role/add');
        Route::get('/role/get_menu_list', 'crm.manage.role/getMenuList');
        Route::post('/role/edit', 'crm.manage.role/edit');
        Route::delete('/role/del', 'crm.manage.role/del');
        // 员工管理
        Route::get('/staff/index', 'crm.manage.staff/index');
        Route::post('/staff/add', 'crm.manage.staff/add');
        Route::post('/staff/edit', 'crm.manage.staff/edit');
        Route::delete('/staff/del', 'crm.manage.staff/del');

    })->middleware(app\cbcrm\middleware\check::class);

    Route::group('trade_tm', function () {
        // 商标交易库
        Route::get('cloudList', 'crm.trade.trademark/cloudList');
        // 本地交易库
        Route::get('localList', 'crm.trade.trademark/localList');
        Route::post('local_add', 'crm.trade.trademark/local_add');
        Route::post('local_edit', 'crm.trade.trademark/local_edit');
        Route::delete('local_del', 'crm.trade.trademark/local_del');
        Route::post('local_sync', 'crm.trade.trademark/local_sync');
        Route::get('getLoaclDetail', 'crm.trade.trademark/getLoaclDetail');
        // 商标交易订单
        Route::get('order/index','crm.trade.trademarkOrder/index');
        Route::post('order/edit','crm.trade.trademarkOrder/edit');
        Route::delete('order/del', 'crm.trade.trademarkOrder/del');

    })->middleware(app\cbcrm\middleware\check::class);

    Route::group('sms',function (){
        // 平台短信接口
        Route::get('platform/init', 'crm.config.platformSms/init');
        // 充值记录
        Route::get('platform/recharge_list', 'crm.config.platformSms/getRechargeList');
        // 创建订单
        Route::get('platform/create_order', 'crm.config.platformSms/createOrder');
        // 获取订单状态
        Route::get('platform/get_order', 'crm.config.platformSms/getOrderStatus');
        // 设置签名
        Route::post('platform/change_sign', 'crm.config.platformSms/changeSign');
        // 申请签名
        Route::post('platform/add_sign', 'crm.config.platformSms/addSign');
        Route::post('platform/del_sign', 'crm.config.platformSms/delSign');
    });

    Route::get('/index/dashboard', 'crm.index/dashboard')->middleware(app\cbcrm\middleware\check::class);

});

Route::get('/web','web/index');
Route::get('/ajax/icon','basic/icon');
Route::get('basic/getRegionTree', 'basic/getRegionTree');// 获取地区树

/**
 * 管理后台
 */
include_once app_path('route').('admin.php');

/**
 * PC网站接口路由
 * */
include_once app_path('route').('site.php');

