<?php


namespace app\cbcrm\services;

use AlibabaCloud\SDK\Dysmsapi\V20170525\Dysmsapi;
use AlibabaCloud\SDK\Dysmsapi\V20170525\Models\SendSmsRequest;
use app\cbcrm\extend\yuntongxun\SendTemplateSMS;
use app\cbcrm\library\qcloudsms\SmsSingleSender;
use app\cbcrm\model\admin\sms\ConfigModel;
use app\cbcrm\model\admin\sms\SignModel;
use app\cbcrm\model\config\SettingModel;
use app\cbcrm\model\config\SmsTemplateModel;
use Darabonba\OpenApi\Models\Config;
use think\Exception;
use think\facade\Log;

/**
 * 短信服务
 * */
class SmsServices
{
    /**
     * @title 分发短信
     * @param string $mobile 短信接收手机号
     * @param array $data 数据内容
     * @param string $sms_title 短信模板标题
     * @param integer $wid 账户ID
     * */
    public function send($mobile, $data,$sms_title,$wid)
    {

        if (!$mobile) {
            return false;
        }
        $channel = (new SettingModel())->getConfigValue('sms','channel', $wid);
        if ($channel == 'custom') {
            // 自配接口
            $templateCode = (new SmsTemplateModel())->getTemplateCode($sms_title, $wid);
            $customConfig = (new SettingModel())->getConfigGroup('sms', $wid);
            $config = $customConfig['engine'][$customConfig['default']];
            $config['default'] = $customConfig['default'];
        }else{
            // 短信数量判断
            $sms_toltal = (new \app\cbcrm\model\admin\store\ConfigModel())->where('wid',$wid)->value('sms_total');
            if (!$sms_toltal) {
                Log::error($wid.'短信余额不足');
                return false;
            }
            // 平台接口
            $templateCode = (new SmsTemplateModel())->getTemplateCode($sms_title, 0);
            $platformConfig = (new ConfigModel())->where('is_default',1)->find();
            if (!$platformConfig) {
                return false;
            }
            $config = json_decode($platformConfig['config'],true);
            $config['default'] = $platformConfig['name'];
            // 签名判断
            $sign = (new SignModel())->where('wid',$wid)->where('is_default',1)->value('sign');
            if ($sign) {
                // 使用自己的签名
                $config['sign'] = $sign;
                $config['signature'] = $sign;
            }
        }

        if (!$templateCode) {
            return false;
        }
        try {
            switch ($config['default']) {
                case 'qcloud':
                    $result = $this->qcloudsms($config, $mobile, $data, $templateCode);
                    if (isset($result['result']) && $result['result'] == 0 && $result['errmsg'] == 'OK') {
                        if($channel == 'platform'){
                            (new \app\cbcrm\model\admin\store\ConfigModel())->where('wid',$wid)->dec('sms_total', 1)->update();
                        }
                        return true;
                    } else {
                        return false;
                    }
                    break;
                case 'aliyun':
                    $result = $this->aliyunsms($config, $mobile, $data, $templateCode);
                    if ($result->body->code == 'OK') {
                        if($channel == 'platform'){
                            (new \app\cbcrm\model\admin\store\ConfigModel())->where('wid',$wid)->dec('sms_total', 1)->update();
                        }
                        return true;
                    } else {
                        return false;
                    }
                    break;
                case 'yuntongxun':
                    $result = $this->ytxsms($config, $mobile,$data, $templateCode);
                    if ($result == NULL) {
                        return false;
                    }
                    if ($result->statusCode != 0) {
                        return false;
                    } else {
                        if($channel == 'platform'){
                            (new \app\cbcrm\model\admin\store\ConfigModel())->where('wid',$wid)->dec('sms_total', 1)->update();
                        }
                        return true;
                    }
                case 'zthysms':
                    $result = $this->zthysms($config, $mobile,$data, $templateCode);
                    if ($result == NULL) {
                        return false;
                    }
                    $result = json_decode($result,true);
                    if ($result['code'] != 200) {
                        return false;
                    } else {
                        if($channel == 'platform'){
                            (new \app\cbcrm\model\admin\store\ConfigModel())->where('wid',$wid)->dec('sms_total', 1)->update();
                        }
                        return true;
                    }
                    break;
            }
        } catch (Exception $e) {
            Log::error('短信发送失败:'.$e->getMessage());
            return false;
        }
    }

    /**
     * @title 发送腾讯云短信
     * @param array $smsconfig 短信配置
     * @param string $mobile 短信接收手机号
     * @param array $data 数据内容
     * @param string $templateCode 短信模板code
     * */
    public function qcloudsms($smsconfig, $mobile, $data,$templateCode)
    {
        $sender = new SmsSingleSender($smsconfig['AppID'], $smsconfig['AppKey']);
        $result = $sender->sendWithParam("86", $mobile, $templateCode, array_values($data), $smsconfig['engine']['qcloud']['sign'], "", "");
        $resp = (array)json_decode($result, true);
        return $resp;
    }

    /**
     * 发送阿里云短信
     * @param array $smsconfig 短信配置
     * @param string $mobile 短信接收手机号
     * @param array $data 数据内容
     * @param string $templateCode 短信模板code
     * */
    public function aliyunsms($smsconfig, $mobile, $data,$templateCode)
    {
        $config = new Config([
            // 您的 AccessKey ID
            "accessKeyId" => $smsconfig['AccessKeyId'],
            // 您的 AccessKey Secret
            "accessKeySecret" => $smsconfig['AccessKeySecret'],
        ]);
        // 访问的域名
        $config->endpoint = "dysmsapi.aliyuncs.com";

        $client = new Dysmsapi($config);
        $sendSmsRequest = new SendSmsRequest([
            "phoneNumbers" => $mobile,
            "templateCode" => $templateCode,
            "templateParam" => json_encode($data, true),
            "signName" => $smsconfig['sign']
        ]);
        $resp = $client->sendSms($sendSmsRequest);

        return $resp;
    }

    /**
     * 发送容联云短信
     * @param array $smsconfig 短信配置
     * @param string $mobile 短信接收手机号
     * @param array $data 数据内容
     * @param string $templateCode 短信模板code
     * */
    public function ytxsms($smsconfig,$mobile,$data,$templateCode)
    {
        $client = new SendTemplateSMS($smsconfig['account_sid'],$smsconfig['account_token'],$smsconfig['app_id']);
        return $client->sendTemplateSMS($mobile,array_values($data),$templateCode);

    }

    /**
     * 发送助通融合云通信短信 https://doc.zthysms.com/web/#/1/13
     * @param array $smsconfig 短信配置
     * @param string $mobile 短信接收手机号
     * @param array $data 数据内容
     * @param string $templateCode 短信模板code
     * */
    public function zthysms($smsconfig,$mobile,$data,$templateCode)
    {
        date_default_timezone_set('PRC'); //设置时区
        $url = "https://api.mix2.zthysms.com/v2/sendSmsTp";
        $tKey     = time();
        $password = md5(md5($smsconfig['password']) . $tKey);
        $data = [
            'username' => $smsconfig['username'], //用户名
            'password' => $password, //接口密码
            'tKey' => $tKey, //tKey
            'tpId'=>$templateCode,
            'signature'=>'【'.$smsconfig['signature'].'】',
            'records'=>[
                [
                    'mobile'=>$mobile,
                    'tpContent'=>$data
                ]
            ]
        ];
        $curl = curl_init(); // 启动一个CURL会话
        curl_setopt($curl, CURLOPT_URL, $url); // 要访问的地址
        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false); // 对认证证书来源的检查
        curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, false); // 从证书中检查SSL加密算法是否存在
        curl_setopt($curl, CURLOPT_USERAGENT, $_SERVER['HTTP_USER_AGENT']); // 模拟用户使用的浏览器
        curl_setopt($curl, CURLOPT_POST, true); // 发送一个常规的Post请求
        curl_setopt($curl, CURLOPT_POSTFIELDS, json_encode($data)); // Post提交的数据包
        curl_setopt($curl, CURLOPT_TIMEOUT, 30); // 设置超时限制防止死循环
        curl_setopt($curl, CURLOPT_HEADER, false); // 显示返回的Header区域内容
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true); // 获取的信息以文件流的形式返回
        curl_setopt($curl, CURLOPT_HEADER, false); //开启header
        curl_setopt($curl, CURLOPT_HTTPHEADER, array(
            'Content-Type: application/json; charset=utf-8'
        )); //类型为json
        $result = curl_exec($curl); // 执行操作
        Log::error('返回结果');
        Log::error($result);
        if (curl_errno($curl)) {
            Log::error(curl_error($curl));
            return null;
        }
        curl_close($curl); // 关闭CURL会话
        return $result;
    }
}
